class hashNode:#散列表的节点
    def __init__(self, key, data):
        self.data = data
        self.key = key
        self.next = None
    def __eq__(self, other):
        return isinstance(other, hashNode) and (other.key==self.key) and (other.data==self.data)

class HashTable:
    def __init__(self, num = 11):#建立一个空的映射表
        self.size = num# 缓存长度
        self.length = 0
        self.slots = [None] * self.size
    def hash_function(self, key):#返回一个键的哈希码。使用取余法
        return key % self.size
    '''
    建议先看Map的具体方法定义，
    再看map具体方法实现过程，
    再看generator的实现方式。
    也即翻到最后从下往上看。
    '''
    # 以下是generator的实现方式
    def _generate(self, pin=None):  # 生成器母函数
        for slot in self.slots:
            while slot is not None:
                yield slot if pin is None else slot.__getattribute__(pin)
                slot = slot.next

    def generate_node(self):  # 返回关于所有节点的生成器
        return self._generate()

    def generate_key(self):  # 返回所有键的生成器
        return self._generate('key')

    def generate_value(self):  # 返回所有值的生成器
        return self._generate('value')

    def generate_hash(self, hash):  # 针对某个hash值进行生成。若hash越界，则抛出IndexError
        if hash >= self.size or hash < 0:
            raise IndexError('No Such Hash code Here!')
        node = self.slots[hash]
        while node is not None:
            yield node
            node = node.next

    def __iter__(self):  # 返回迭代器，迭代每一个节点对象
        return self.generate_node()

    #以下是Map具体方法的实现过程
    def put(self, key, data):#放入一个数据
        hash = self.hash_function(key)
        for node in self.generate_hash(hash):
            if node.key == key:#已经有key存在
                node.data = data#刷新覆盖数据，长度不变
                return
        # 遍历完毕，若发现没有元素，则把元素添加在开头
        node = hashNode(key, data)
        node.next = self.slots[hash]
        self.slots[hash] = node
        self.length+=1 #新增一个数据，长度+1
        return
    def get(self, key):#取出一个数据，如果没有这个数据，抛出KeyError
        hash = self.hash_function(key)
        for node in self.generate_hash(hash):
            if node.key == key:
                return node.data
        raise KeyError(key)
    def pop(self, key):#取出并移除一个数据。如果没有这个数据，抛出KeyError
        hash = self.hash_function(key)
        if (self.slots[hash] is not None) and (self.slots[hash].key == key):#特殊判断第一个元素是否为需要删去的元素
            node = self.slots[hash]
            self.slots[hash] = node.next
            self.length -= 1
            return node.data
        for node in self.generate_hash(hash):
            if (node.next is not None) and (node.next.key == key):#之所以要下一个，是因为需要连接前一个
                del_node = node.next
                node.next = node.next.next
                self.length -= 1
                return del_node.data
        raise KeyError(key)

    #以下对Map的具体方法(除了Map())进行定义
    def __setitem__(self, key, data):#设置散列表的值 #使m[X]=XX可行
        self.put(key, data)
    def __getitem__(self, key):#获取散列表的值。#使print(m[X])可行
        try:
            return self.get(key)
        except KeyError as e:#不存在时，抛出KeyError
            raise e
    def __delitem__(self, key):#删除某一个键 #使del m[X]可行
        try:
            return self.pop(key)
        except KeyError as e:#不存在时抛出KeyError
            raise e
    def __len__(self):#返回已经占用的长度
        return self.length
    def __contains__(self, key):#判断键是否在map中 #使 in 可行
        return key in self.generate_key()
    def __str__(self):#返回hashmap的字符串表示形式。借用字典的str方法 #使print(m)可行
        return 'dict_values(%s)' % ([(node.key,node.data) for node in self])

#测试代码
hash = HashTable()
print(hash)
hash.put(0,"o")
hash.put(1,"a")
hash.put(2,"b")
hash.put(3,"c")
hash.put(4,"d")
hash.put(5,"e")
hash.put(6,"f")
print(hash)
print(len(hash))

print(1 in hash)

del hash[5]
print(hash)
print(len(hash))