#1. UnorderedList的方法
class Node: #定义节点对象，链表实现的最基本元素
    def __init__(self, initdata): #初始化
        self.data = initdata
        self.next = None

    def getData(self):
        return self.data

    def getNext(self):
        return self.next

    def setData(self, newdata):
        self.data = newdata

    def setNext(self, newnext):
        self.next = newnext

class UnorderedList: #定义无序列表
    def __init__(self):
        self.head = None #初始化空表

    def add(self, item):
        temp = Node(item)
        temp.setNext(self.head)
        self.head = temp

    def isEmpty(self): #head指向None时为空表
        return self.head == None

    def size(self):
        current = self.head
        count = 0
        while current != None:
            count = count + 1
            current = current.getNext()
        return count

    def search(self, item):
        current = self.head
        found = False
        while current != None and not found:
            if current.getData() == item:
                found = True
            else:
                current = current.getNext()
        return found

    def __getitem__(self, num):
        current = self.head
        n = 0
        while n != num:
            current = current.getNext()
            n = n + 1
        return current.getData()

    def remove(self, item): #删除item的节点
        current = self.head
        previous = None #维护current的前一个节点
        found = False
        while not found:
            if current.getData() == item:
                found = True
            else:
                previous = current
                current = current.getNext()
        if previous == None:
            self.head = current.getNext()
        else:
            previous.setNext(current.getNext)#前一个节点的next指向current的下一个节点即删除

    def append(self, item):
        temp = Node(item)
        current = self.head
        previos = None
        if current == None: #空列表的特殊情况
            self.head = temp
            temp.setNext(None)
        else:
            while current != None:
                previous = current
                current = current.getNext()#遍历到表尾
            previous.setNext(temp)
            temp.setNext(None)

    def index(self, item): #返回item的首次出现位置
        current = self.head
        found = False
        num = 0 #num记录current的位置
        while current != None and not found:
            if current.getData() != item:
                current = current.getNext()
                num = num + 1
            else:
                found = True
        if found == True: #当找到item时返回值
            return num + 1
        else:
            return None

    def pop(self, index): #删除index位置的节点
        current = self.head
        previous = None
        num = 0 #num标记current的位置
        while index != None and num != index:
            previous = current
            current = current.getNext()
            num = num + 1
        if index == None or previous == None: #pop（）和空列表的特殊情况
            self.head = current.getNext()
        else:
            previous.setNext(current.getNext())

    def insert(self, pos, item): #在pos位置插入item
        temp = Node(item)
        current = self.head
        num = 0 #num标记current的位置
        previous = None
        while pos != num:
            previous = current
            current = current.getNext()
            num = num + 1
        if previous == None: #空列表特殊情况
            temp.setNext(current)
            self.head = temp
        else:
            temp.setNext(current)
            previous.setNext(temp)

    def __str__(self):
        l = ""
        current = self.head
        while current != None: #遍历，把每个节点转为str
            l = l + str(current.getData())
            current = current.getNext()
        return l

    def __getslice__(self, i, j):  #位置i、j之间的子序列
        num = 0
        l = ''
        current = self.head
        while num != i:
            current = current.getNext()
            num = num + 1
        while num < j and current != None: #循环初始num=i
            l = l + str(current.getData())
            current = current.getNext()
            num = num + 1
        return l

#2. 将OrederedList作为UnorderedList的子类来实现，isEmpty/size/remove与节点的次序无关，实现与UnorderedList一致
class OrderedList(UnorderedList):
    def search(self, item):
        current = self.head
        found = False
        stop = False
        while current != None and not found and not stop:
            if current.getData() == item:
                found = True
            else:
                if current.getData() > item: #当前节点数据项大于要查找的数据项，停止继续查找；链表节点有序排列特性
                    stop == True
                else:
                    current = current.getNext()
        return found

    def add(self,item): #维护列表的有序性，需找到合适的位置插入
        current = self.head
        previous = None
        stop = False
        while current != None and not stop:
            if current.getData() > item:
                stop = True
            else:
                previous = current
                current = current.getNext()

        temp = Node(item)
        if previous == None:
            temp.setNext(self.head)
            self.head = temp
        else:
            temp.setNext(current)
            previous.setNext(temp)

    def index(self,item): #列表的有序性因此不需要遍历
        current = self.head
        found = False
        stop = False
        num = 0 #num记录current的位置
        while current != None and not found and not stop:
            if current.getData() == item:
                found = True
            else:
                if current.getData() > item:
                    stop = True
                else:
                    current = current.getNext()
                    num = num + 1
        if found == True: #当找到item时返回值
            return num + 1
        else:
            return None

#3. 采用列表实现ADT Stack & Queue
class Stack:
    def __init__(self):
        self.items = UnorderedList()
    def isEmpty(self):
        return self.items.isEmpty()
    def push(self,item):
        self.items.add(item) #加入表头
    def pop(self):
        return self.items.pop() # pop无序列表的第一个节点
    def peek(self):
        return self.items.head.getData() #无序列表的head指向第一个节点
    def size(self):
        return self.items.size()

class Queue:
    def __init__(self):
        self.items = UnorderedList()
    def isEmpty(self):
        return self.items.isEmpty()
    def enqueue(self,item):  #加入表尾
        self.items.append(item)
    def dequeue(self):
        return self.items.pop()
    def size(self):
        return self.items.size()











