class Node:
    def __init__(self, initkey, initvalue):
        self.key = initkey
        self.value = initvalue
        self.next = None

    def setkey(self, newkey):
        self.key = newkey

    def setvalue(self, newvalue):
        self.value = newvalue

    def getkey(self):
        return self.key

    def getvalue(self):
        return self.value

    def setnext(self,newnext):
        self.next = newnext

    def getnext(self):
        return self.next

# 以上实现Node结构的基本属性和功能，节点中有三个属性：key、value和next

class Hashtable:

    def __init__(self, tablesize):
        # 设置散列表的大小
        self.size = tablesize
        # 用slots存键值对
        self.slots = [None] * self.size

    def hashfunction(self, item):
        # 判断，如果item是正整数，那么返回其对散列表大小的余数
        if isinstance(item, int):
            return item % self.size
        # 如果是字符串，那么返回各个位置上字符对应的整数的加权和
        elif isinstance(item, str):
            sum = 0
            for j in range(len(item)):
                sum += (j + 1) * ord(item[j])
            return sum % self.size

    def put(self, key, value):
        # 先得到key的散列值
        hashvalue = self.hashfunction(key)
        # 如果该散列位置是空的，那么加上一个节点对
        if self.slots[hashvalue] == None:
            self.slots[hashvalue] = Node(key, value)
        else:
            temp = Node(key, value)
            current = self.slots[hashvalue]
            previous = None
            while current != None and current.getkey() != key:
                previous = current
                current = current.getnext()
            # key已存在，替换其对应的值
            if current.key == key:
                current.setvalue(value)
            # key不存在，在self.slots最后分别加上tempkey
            else:
                previous.setnext(temp)

    def get(self, key):
        # 先得到key的散列值
        hashvalue = self.hashfunction(key)
        # 在其对应的链表中搜索key
        current = self.slots[hashvalue]
        # 只要currentkey的值不是key，就将currentkey指向下一个节点
        while current.getkey() != key:
            current = current.getnext()
        # 返回对应的value值
        return current.getvalue()

    def __getitem__(self, item):
        return self.get(item)

    def __setitem__(self, key, value):
        self.put(key, value)

    def __delitem__(self, key):
        # 先得到key的散列值
        hashvalue = self.hashfunction(key)
        # 在其对应的链表中搜索key
        current = self.slots[hashvalue]
        previous = None
        while current.getkey() != key:
            previous = current
            current = current.getnext()
        # 删除currentkey位置：如果要删除的不是第一个，那么改变previous的next；如果是第一个那么改变self.slots[hashvalue]的值
        if previous != None:
            previous.setnext(current.getnext())
        else:
            self.slots[hashvalue] = current.getnext()

    def __len__(self):
        # 用length记录长度，初始为0
        length = 0
        # 用current历遍每个节点
        for i in range(self.size):
            current = self.slots[i]
            while current != None:
                length += 1
                current = current.getnext()
        return length

    def __index__(self, key):
        # 先得到key的散列值
        hashvalue = self.hashfunction(key)
        # 在其对应的链表中搜索key
        current = self.slots[hashvalue]
        while current != None:
            if current.getkey() == key:
                return True
            else:
                current = current.getnext()
        return False

H = Hashtable(11)
H['12'] = "you"
print(len(H))
print(H['12'])
del H['12']
print(len(H))
