# 增加属性最大负载因子
# 每次put之前先检验是否达到最大负载，达到则将size加倍
# 并将原先的所有键值对“倒”进新的散列表
# 然后继续put
# 其它方法不变



class HashTable:
    def __init__(self, tableSize, maxindex):
        self.maxindex = maxindex
        self.size = tableSize
        self.slots = [None] * tableSize
        self.values = [None] * tableSize

    def hashfunction(self, key):
        # 判断，如果item是正整数，那么返回其对散列表大小的余数
        if isinstance(key, int):
            return key % self.size
        # 如果是字符串，那么返回各个位置上字符对应的整数的加权和
        elif isinstance(key, str):
            sum = 0
            for j in range(len(key)):
                sum += (j + 1) * ord(key[j])
            return sum % self.size

    def rehash(self, hashvalue):
        return hashvalue + 1

    def put(self, key, value):

        # 判断负载因子是否超过maxindex，
        if len(self) / self.size > self.maxindex:
            # 暂存所有键值对
            templist = []
            for i in range(0,self.size-1):
                if self.slots[i] != None:
                    templist.append([self.slots[i], self.values[i]])
            # 将self扩大一倍
            self.size *= 2
            # 格式化self
            self.slots = [None] * self.size
            self.values = [None] * self.size
            # 重载原始数据
            for x in templist:
                self.put(x[0], x[1])


        # 继续加入新数据
        hashvalue = self.hashfunction(key)
        if self.slots[hashvalue] == None:
            self.slots[hashvalue] = key
            self.values[hashvalue] = value
        else:

            if self.slots[hashvalue] == key:
                self.values[hashvalue] = value
            else:

                nextslot = self.rehash(hashvalue)
                while self.slots[nextslot] != None and \
                    self.slots[nextslot] != key:
                    nextslot = self.rehash(nextslot)

                if self.slots[nextslot] == None:
                    self.slots[nextslot] = key
                    self.values[nextslot] = value
                else:
                    self.values[nextslot] = value

    def get(self, key):
        startslot = self.hashfunction(key)

        found = False
        stop = False
        value = None
        position = startslot
        while self.slots[position] != key and \
                not found and not stop:
            if self.slots[position] == key:
                found = True
                value = self.values[position]
            else:
                position = self.rehash(position)
                if position == startslot:
                    stop = True
        return value

    def __getitem__(self, key):
        return self.get(key)

    def __setitem__(self, key, value):
        self.put(key, value)

    def __len__(self):
        return self.size - self.slots.count(None)

    def __index__(self, key):
        startslot = self.hashfunction(key)

        found = False
        stop = False
        position = startslot

        while self.slots[position] != key and  \
            not found and not stop:

            if self.slots[position] == key:
                found = True
            else:
                position = self.rehash(position)
                if position == startslot:
                    stop = True

        return found

    def __delitem__(self, key):

        position = self.hashfunction(key)
        # 锁定该键值对
        while self.slots[position] != key:
            position = self.rehash(position)
        # 删除该键值对
        self.slots[position] = None
        self.values[position] = None


