# 产生一个大小为1000000的随机正整数数列表作为待排序的列表
# 未改造的快排：每次取待排范围中的第一个数作为中值
# 改造后的快排：
#   确定中值：取出初始的最大值和最小值，产生一个和原列表一样长（或多一个）的等差递增参考列表；
#   每次取参考列表中对应范围的第一个和最后一个元素的平均值作为中值；
#   其他部分同标准快排，实现排序；
# 分别使用两种快排方法来排序，用timeit计时。
# 比较两者花费的时间

from timeit import Timer
import random


# qsort1将待排序的第一个数字作为中值
def qsort1(alist):

    qsorthelper1(alist, 0, len(alist)-1)

def qsorthelper1(alist, first, last):

    if first < last:
        splitvalue = partition1(alist, first, last)

        qsorthelper1(alist, first, splitvalue-1)
        qsorthelper1(alist, splitvalue+1, last)

def partition1(alist, first, last):

    if first < last:

        pivotvalue = alist[first]
        leftmark = first+1
        rightmark = last

        done = False
        while leftmark <= rightmark and not done:


            while leftmark <= rightmark and alist[leftmark] <= pivotvalue:
                leftmark += 1
            while rightmark >= leftmark and alist[rightmark] >= pivotvalue:
                rightmark -= 1

            if leftmark > rightmark:
                done = True
            else:
                alist[leftmark], alist[rightmark] = alist[rightmark], alist[leftmark]

        alist[first], alist[rightmark] = alist[rightmark], alist[first]
        return rightmark

# qsort2借用参考列表产生中值
def qsort2(alist):
    # 设计一个标准列表stdlist作为计算中值的参考
    Min = min(alist)
    Max = max(alist)
    d = (Max - Min) // len(alist)
    if d == 0 :
        d += 1
    stdlist = list(range(Min, Max+d, d))

    qsorthelper2(alist, 0, len(alist)-1, stdlist)

def qsorthelper2(alist, first, last, stdlist):

    if first < last:
        splitvalue = partition2(alist, first, last, stdlist)

        qsorthelper2(alist, first, splitvalue, stdlist)
        qsorthelper2(alist, splitvalue+1, last, stdlist)

def partition2(alist, first, last, stdlist):

    if first < last:

        pivotvalue = (stdlist[first]+stdlist[last])//2

        leftmark = first
        rightmark = last

        done = False
        while leftmark <= rightmark and not done:


            while leftmark <= rightmark and alist[leftmark] <= pivotvalue:
                leftmark += 1
            while rightmark >= leftmark and alist[rightmark] > pivotvalue:
                rightmark -= 1

            if leftmark > rightmark:
                done = True
            else:
                alist[leftmark], alist[rightmark] = alist[rightmark], alist[leftmark]

        return rightmark



# 使用一个大小为10^5的数组，将数组随机打乱
mylist = list(range(1, 10**5))
random.shuffle(mylist)

# 保证两种排序使用的是同样的初始列表
lista = mylist.copy()
listb = mylist.copy()


def testqsort1():
    qsort1(lista)
def testqsort2():
    qsort2(listb)

# 两种排序都测试30组数据，比较两种排序的总时长
t1 = Timer("testqsort1()", "from __main__ import testqsort1")
t2 = Timer("testqsort2()", "from __main__ import testqsort2")


print("testqsort1花费了%f秒" % (t1.timeit(number=1)))
print("testqsort2花费了%f秒" % (t2.timeit(number=1)))

# 结果表明，至少在前10^5个数的排序上，大多数情况下将第一个数作为中值比用辅助列表产生中值的排序要快