# 实现 UnorderedList 的append, index, pop, insert, __str__, __getitem__ 的方法

class Node:   # 定义类：节点Node
    def __init__(self,initdata):     # 包括数据项本身信息，以及指向下一个节点的引用信息
        self.data = initdata
        self.next = None   # next为None时表示没有下一个节点了

    def getData(self):   # 定义getData获取自身数据方法
        return self.data

    def getNext(self):    # 定义getNext获取下一个节点数据方法
        return self.next

    def setData(self,newdata):    # 定义setData给自身赋值方法
        self.data = newdata

    def setNext(self,newnext):   # 定义setNext给下一个节点赋值方法
        self.next = newnext

class UnorderedList:   # 定义类：无序列表

    def __init__(self):
        self.head = None   # 设立属性head，保存对第一个节点的引用，空表的head为None

    def isEmpty(self):   # 定义isEmpty方法，判断列表是否为空
        return self.head == None

    def size(self):   # 定义size方法，求列表长度
        current = self.head
        count = 0
        while current != None:   # 遍历列表
            count += 1
            current = current.getNext()
        return count   # 返回列表长度

    def append(self,item):   # 定义append方法
        temp = Node(item)
        if self.isEmpty():
            self.head = temp   # 若为空列表，将添加的元素设为第一个元素
        else:
            current = self.head
            while current.getNext() != None:
                current = current.getNext()   # 遍历列表
            current.setNext(temp)    # 将原列表最后一位指向新数据项

    def index(self,item):   # 定义index方法
        if self.isEmpty():
            print('UnorderedList is empty.')   # 若为空列表，直接输出'列表为空'
            return

        current = self.head
        count = 0
        found = None
        while current != None and not found:   # 未找到时，遍历列表
            if current.getData() == item:   # 找到后将found记录为True，终止遍历列表
                found = True
            else:
                current = current.getNext()   # 没找到时继续
                count += 1
        if found:
            return count   # 找到以后，将对应的索引号输出
        else:
            return None   # 没找到时，返回None

    def pop(self,pos):   # 定义pop方法
        if pos >= self.size():
            raise IndexError('pos out of bounds')   # 如果pos超出列表范围，返回'索引错误'
        else:
            i = 0
            current = self.head
            while i+1 < pos:   # 遍历列表，直到索引位置
                current = current.getNext()
                i += 1
            if current.getNext() == None or pos == 0:   # 列表中只有1位，或者是删除第1位的情况
                self.head = current.getNext()
                return current.getData()    # 返回第1位元素
            else:
                temp = current.getNext().getData()
                current.setNext(current.getNext().getNext())   # 将current直接指向下下位，相当于pop了current的下一位
                return temp  # 返回current的下一位，即pos位对应的元素

    def insert(self,pos,item):  # 定义insert方法
        i = 0
        temp = Node(item)   # 创建一个新节点
        current = self.head
        while i+1 != pos:   # 遍历列表，直到找到赋值位置
            current = current.getNext()
            i += 1
        temp.setNext(current.getNext())    # 将新节点指向赋值位置的节点
        current.setNext(temp)  # 将赋值位置前一位的节点指向新节点

    def __str__(self):   # 定义列表以字符串形式输出的str方法
        if self.isEmpty():
            print('UnorderedList is empty.')   # 若为空列表，直接输出'列表为空'
            return
        i = 0
        s = '['
        current = self.head
        while i+1 < self.size():   # 遍历列表
            s = s + repr(current.getData()) +','   # 将列表中的每一位转为字符串数据格式，以','隔开
            current = current.getNext()
            i += 1
        s = s + repr(current.getData()) + ']'
        return s   # 输出字符串

    def __getitem__(self, item):   # 定义getitem方法
        if isinstance(item,int):   # 如果传入的参数为整数
            i = 0
            current = self.head
            while i < item:   # 遍历列表，直到索引号
                current = current.getNext()
                i += 1
            return current.getData()   # 返回索引位置对应的值
        if isinstance(item,slice):   # 如果传入的参数是一个切片对象
            start = item.start
            stop = item.stop   # 将切片两端的值记为start和stop
            i = 0
            j = 1
            current = self.head
            l = []
            while i < start:   # 先找到切片开始处对应的值，加到列表l中
                current = current.getNext()
                i += 1
            l.append(current.getData())
            while j < stop - start:   # 遍历切片包含的位数，找到相应的值，加到列表l末尾
                current = current.getNext()
                l.append(current.getData())
                j += 1
            return l   # 返回列表l


#  测试部分

mylist = UnorderedList()   # 创建无序列表
mylist.append('a')   # 添加'a'，列表变为['a']
print(mylist.isEmpty())   # 输出结果为False
mylist.append('b')   # 添加'b'，列表变为['a','b']
print(mylist)  # 输出结果为['a','b']
print(mylist.size())   # 输出列表长度为2
print(mylist.index('b'))   # 索引元素'b'，输出结果为1
print(mylist.index('c'))   # 输出结果为None
print(mylist.pop(1))   # 删除第2位，列表变为['a']，输出结果为'b'
print(mylist)  # 输出结果为['a']
print(mylist.size()) # 列表长度为1
print(mylist.index('b'))   # 输出结果为None
mylist.append('c')
mylist.append('d') # 列表变为['a','c','d']
print(mylist)  # 输出结果为['a','c','d']
print(mylist.pop(0))   # 删除第1位，列表变为['c','d']，输出结果为'a'
print(mylist.size())   # 输出列表长度为2
mylist.insert(1,'e')  # 在第2位插入元素'e'，列表变为['c','e','d']
print(mylist)  # 输出结果为['c','e','d']
print(mylist.index('e'))   # 索引元素'e'，输出结果为1
print(mylist.size())  # 输出列表长度位3
print(mylist)   # 输出结果为['c','e','d']
mylist.append('f')
mylist.append('g')
mylist.append('h')  # 列表变为['c','e','d','f','g','h']
print(mylist)   # 输出结果为['c','e','d','f','g','h']
print(mylist[3])   # 输出结果为f
print(mylist[2:5])   # 输出结果为['d','f','g']















