# 将 OrderedList 作为 UnorderedList 的子类来实现

class Node:   # 定义类：节点Node
    def __init__(self,initdata):     # 包括数据项本身信息，以及指向下一个节点的引用信息
        self.data = initdata
        self.next = None   # next为None时表示没有下一个节点了

    def getData(self):   # 定义getData获取自身数据方法
        return self.data

    def getNext(self):    # 定义getNext获取下一个节点数据方法
        return self.next

    def setData(self,newdata):    # 定义setData给自身赋值方法
        self.data = newdata

    def setNext(self,newnext):   # 定义setNext给下一个节点赋值方法
        self.next = newnext

class UnorderedList:   # 定义UnorderedList类
    def __init__(self):
        self.head = None   # 设立属性head，保存对第一个节点的引用，空表的head为None

    def isEmpty(self):   # 定义isEmpty方法，判断列表是否为空
        return self.head == None

    def size(self):   # 定义size方法，求列表长度
        current = self.head
        count = 0
        while current != None:   # 遍历列表
            count += 1
            current = current.getNext()
        return count   # 返回列表长度

    def search(self,item):   # 定义父类中的search方法，查找某数据是否在列表中（需要遍历列表查找）
        current = self.head
        found = False
        while current != None and not found:   # 遍历列表
            if current.getData() == item:
                found = True     # 找到要查到的数据，终止循环
            else:
                current = current.getNext()
        return found

    def add(self,item):   # 定义父类中的add方法，向列表中添加元素（无序表元素添加在表头）
        temp = Node(item)
        temp.setNext(self.head)
        self.head = temp

    def remove(self,item):   # 定义remove方法，移除某元素
        current = self.head
        previous = None
        found = False
        while not found:   # 遍历列表，直到找到item对应位置
            if current.getData() == item:
                found = True
            else:
                previous = current
                current = current.getNext()
        if previous == None:   # 列表中只有1个元素的情况
            self.head = current.getNext()   # head指向current的下个节点，相当于删除了第一个节点
        else:   # 列表中不止1个元素，previous直接指向current的下一个节点，相当于删除了current这个节点
            previous.setNext(current.getNext())

    def __str__(self):   # 定义列表以字符串形式输出的str方法 (方便测试显示)
        if self.isEmpty():
            print('UnorderedList is empty.')   # 若为空列表，直接输出'列表为空'
            return
        i = 0
        s = '['
        current = self.head
        while i+1 < self.size():   # 遍历列表
            s = s + repr(current.getData()) + ','   # 将列表中的每一位转为字符串数据格式，以','隔开
            current = current.getNext()
            i += 1
        s = s + repr(current.getData()) + ']'
        return s   # 输出字符串

class OrderedList(UnorderedList):   # 定义OrderedList子类

    def search(self,item):   # 子类重新定义search方法（不需要遍历全部元素，根据有序性，循环到数据项大于要查找数据即可停止）
        current = self.head
        found = False
        stop = False
        while current != None and not found and not stop:  # 遍历列表，直到找到要查找的数据项，或循环到的数据项已经大于要查找数据项
            if current.getData() == item:
                found = True    # 找到要查找的数据项，found赋值为True，终止循环
            else:
                if current.getData() > item:
                    stop = True   # 循环到的数据项大于要查找数据项，stop赋值为True，终止循环
                else:
                    current = current.getNext()   # 继续查找
        return found

    def add(self,item):   # 子类重新定义add方法（由于有序性，add需要添加在合适位置，以维护整个列表的有序性）
        current = self.head
        previous = None
        stop = False
        while current != None and not stop:   # 发现插入位置
            if current.getData() > item:
                stop = True
            else:
                previous = current
                current = current.getNext()
        temp = Node(item)
        if previous == None:   # 插入在表头
            temp.setNext(self.head)
            self.head = temp
        else:   # 插入在表中
            temp.setNext(current)
            previous.setNext(temp)


# 测试部分

mylist = OrderedList()   # 创建有序列表
print(mylist.isEmpty())   # 检查列表是否为空，输出结果True
mylist.add('b')   # 列表中加入'b'元素
print(mylist.isEmpty())   # 检查列表是否为空，输出结果False
print(mylist.size())   # 输出列表长度为1
mylist.add('f')   # 列表中加入'f'元素
print(mylist)   # 输出结果为['b','f']
mylist.add('a')   # 列表中加入'a'元素
mylist.add('e')   # 列表中加入'e'元素
mylist.add('c')   # 列表中加入'c'元素
print(mylist)   # 输出结果为['a','b','c','e','f']
mylist.remove('b')   # 移除'b'元素
print(mylist.size())   # 输出列表长度为4
print(mylist)   # 输出结果为['a','c','e','f']
print(mylist.search('b'))   # 查找'b'，输出结果为False
print(mylist.search('c'))  # 查找'c'，输出结果为True



