# 采用链表实现 ADT Stack 和 ADT Queue

class Node:   # 定义类：节点Node
    def __init__(self,initdata):     # 包括数据项本身信息，以及指向下一个节点的引用信息
        self.data = initdata
        self.next = None   # next为None时表示没有下一个节点了

    def getData(self):   # 定义getData获取自身数据方法
        return self.data

    def getNext(self):    # 定义getNext获取下一个节点数据方法
        return self.next

    def setData(self,newdata):    # 定义setData给自身赋值方法
        self.data = newdata

    def setNext(self,newnext):   # 定义setNext给下一个节点赋值方法
        self.next = newnext


class Stack():   # 定义类：栈
    def __init__(self):
        self.head = None

    def isEmpty(self):   # 定义isEmpty方法，判断栈是否为空
        return self.head == None

    def push(self,item):   # 将item项加入栈顶（以表头作为栈顶）
        temp = Node(item)
        temp.setNext(self.head)
        self.head = temp

    def pop(self):  # 定义pop方法，移除栈顶数据项，栈被修改
        temp = self.head
        self.head = temp.getNext()
        return temp.getData()

    def peek(self):   # 定义peek方法，返回栈顶数据项，栈不变
        return self.head.getData()

    def size(self):   # 定义size方法，返回栈中有多少个数据项
        current = self.head
        count = 0
        while current != None:   # 遍历栈
            count += 1
            current = current.getNext()
        return count   # 返回栈长度


class Queue:   # 定义类：队列
    def __init__(self):
        self.head = None

    def isEmpty(self):   # 定义isEmpty方法，判断栈是否为空
        return self.head == None

    def enqueue(self,item):  # 定义enqueue方法，将数据项添加到队尾（以表尾作为队尾），无返回值
        temp = Node(item)
        if self.isEmpty():
            self.head = temp   # 若队列为空，将添加的元素设为第一个元素
        else:
            current = self.head
            while current.getNext() != None:
                current = current.getNext()   # 遍历列表
            current.setNext(temp)    # 将原队列最后一个节点指向新数据项

    def dequeue(self):   # 定义dequeue方法，从队首移除数据项，返回值为队首数据项，队列被修改（以表头作为队首）
        temp = self.head
        self.head = temp.getNext()
        return temp.getData()

    def size(self):   # 定义size方法，返回队列中有多少个数据项
        current = self.head
        count = 0
        while current != None:   # 遍历队列
            count += 1
            current = current.getNext()
        return count   # 返回队列长度




# 测试部分

# 栈测试
l1 = Stack()   # 创建空栈
print(l1.isEmpty())   # 返回True，栈为空
l1.push('a')   # 将元素'a'压入栈顶
print(l1.isEmpty())   # 返回False，栈不空
l1.push('b')   # 将元素'b'压入栈顶
print(l1.peek())   # 返回栈顶元素'b'
print(l1.pop())   # 移除栈顶元素，返回栈顶元素'b'
print(l1.peek())   # 返回栈顶元素'a'
l1.push('c')   # 将元素'c'压入栈顶
l1.push('e')   # 将元素'e'压入栈顶
print(l1.peek())   # 返回栈顶元素'e'
print(l1.size())   # 返回栈的长度为3

print()

# 队列测试
l2 = Queue()  # 创建空队列
print(l2.isEmpty())  # 返回True，队列为空
l2.enqueue('a')   # 将元素'a'添加至队尾
print(l2.isEmpty())   # 返回False，队列不空
l2.enqueue('b')   # 将元素'b'添加至队尾
print(l2.size())   # 返回队列长度为2
l2.enqueue('c')   # 将元素'c'添加至队尾
print(l2.dequeue())   # 移除队首元素，返回队首元素'a'
l2.enqueue('c')   # 将元素'c'添加至队尾
l2.enqueue('e')   # 将元素'e'添加至队尾
print(l2.size())   # 返回队列的长度为4
print(l2.dequeue())   # 移除队首元素，返回队首元素'b'






