# 实现双链表结构的 UnorderedList

class Node:   # 定义类：节点Node
    def __init__(self,initdata):     # 包括数据项本身信息，以及指向前一个节点和下一个节点的引用信息
        self.data = initdata
        self.next = None   # 引用下一个节点
        self.prev = None   # 引用前一个节点

    def getData(self):   # 定义getData获取自身数据方法
        return self.data

    def getNext(self):    # 定义getNext获取下一个节点数据方法
        return self.next

    def getPrev(self):    # 定义getPrev获取前一个节点数据方法
        return self.prev

    def setData(self,newdata):    # 定义setData给自身赋值方法
        self.data = newdata

    def setNext(self,newnext):   # 定义setNext给下一个节点赋值方法
        self.next = newnext

    def setPrev(self,newprev):   # 定义setPrevt给前一个节点赋值方法
        self.prev = newprev

class UnorderedList:   # 定义类：无序列表
    def __init__(self):   # 初始化
        self.head = None   # 设立属性head，保存对第一个节点的引用，空表的head为None
        self.tail = None   # 设立属性tail，保存对最后一个节点的引用，空表的tail为None

    def isEmpty(self):   # 定义isEmpty方法，判断列表是否为空
        return self.head == None

    def size(self):   # 定义size方法，求列表长度
        while not self.isEmpty():
            count = 1
            current = self.head
            while current != self.tail:   # 遍历列表
                count += 1
                current = current.getNext()
            return count
        else:
            return 0

    def add(self,item):   # 定义add方法，向列表中添加元素，添加在表头
        if not self.isEmpty():   # 列表不为空，修改head指向的元素
            temp = Node(item)
            temp.setNext(self.head)
            self.head = temp
        else:   # 列表为空，head和tail指向的元素都被修改
            self.head = self.tail = Node(item)

    def append(self, item):   # 定义append方法，追加数据，追加在表尾
        if not self.isEmpty():   # 列表不为空，修改tail指向的元素
            temp = Node(item)
            self.tail.setNext(temp)
            self.tail = temp
        else:   # 列表为空，head和tail指向的元素都被修改
            self.head = self.tail = Node(item)

    def __str__(self):   # 定义列表以字符串形式输出的str方法 (方便测试显示)
        if self.isEmpty():
            print('UnorderedList is empty.')   # 若为空列表，直接输出'列表为空'
            return
        i = 0
        s = '['
        current = self.head
        while i+1 < self.size():   # 遍历列表
            s = s + repr(current.getData()) + ','   # 将列表中的每一位转为字符串数据格式，以','隔开
            current = current.getNext()
            i += 1
        s = s + repr(current.getData()) + ']'
        return s   # 输出字符串

    # 其余方法，如search,index,insert,remove,pop,__getitem__的实现与5.1,5.2中相同即可

# 测试部分

mylist = UnorderedList()
print(mylist.isEmpty())   # 输出结果为True，表为空
mylist.append('a')   # 表尾添加数据'a'
print(mylist.isEmpty())    # 输出结果为False,表不空
mylist.add('b')   # 表头添加数据'b'
print(mylist)   # 输出结果为['b','a']
mylist.append('c')   # 表尾添加数据'c'
print(mylist.size())   # 输出列表长度为3
mylist.append('d')   # 表尾添加数据'd'
mylist.add('e')   # 表头添加数据'e'
print(mylist)   # 输出结果为['e','b','a','c','d']