class DeNode:
    def __init__(self,initdata):
        # 双向节点有三个基本量，值，前一个节点的指向，后一个节点的指向，两个指向都初始为None
        self.data = initdata
        self.prev = None
        self.next = None
    def setdata(self,newnext):
        self.data = newnext
    def getdata(self):
        return self.data
    def setnext(self,newdata):
        self.next = newdata
    def getnext(self):
        return self.next
    def setprev(self,newprev):
        self.prev = newprev
    def getprev(self):
        return self.prev
# 以上实现双向节点的基本功能
# 链表无序列表表类
class UnorderedList:

    def __init__(self):
        # 创建一个空链表，head用来指向第一个节点，tail用来指向最后一个节点，head和tail都初始为None
        self.head = None
        self.tail = None

    def append(self,item):
        # temp为待加入的新节点
        temp = DeNode(item)
        # 如果列表一开始是空的，那么head和tail都指向新节点
        if self.head == None:
            self.head = temp
            self.tail = temp
        # 否则只对tail做变更
        else:
            self.tail.setnext(temp)
            temp.setprev(self.tail)
            self.tail = temp

    def size(self):
        # size存列表的大小，初始为0
        size = 0
        # current历遍所有的节点，每指向一个新的节点，size加一
        current = self.head
        while current != None:
            current = current.getnext()
            size += 1
        return size

    def index(self,item):
        # current指向待检验的节点，初始为链表头，历遍所有节点
        current = self.head
        # index存current的位置
        index = 0
        # 只要current不是None那么检验current的值是否为item
        while current != None:
            # 找到则返回index
            if current.getdata() == item:
                return index
            # 否则current移到下一个节点
            else:
                current = current.getnext()
                index += 1
        # 若没有找到，则打印提示“item不在列表中”
        print("%s is not in this UnorderedList"%(item))

    def insert(self,pos,item):
        # temp为待插入的节点
        temp = DeNode(item)
        # 如果列表一开始是空的，那么只能插在0号位置
        if self.head == None:
            self.append(item)
        # 不为空情况
        else:
            pos = pos % self.size()
            # 如果插入位置为0，那么只对链表头做变更
            if pos == 0:
                temp.setnext(self.head)
                self.head.setprev(temp)
                self.head = temp
            # 如果插入位置为列表尾，那么只对链表尾做变更
            elif pos == (self.size()):
                self.tail.setnext(temp)
                temp.setprev(self.tail)
                self.tail = temp
            # 否则
            else:
                # 用current指向待插入位置的前一个节点
                current = self.head
                for i in range(0,pos-1):
                    current = current.getnext()
                # 对current、current的下一个节点、temp的前后指向做变更
                temp.setnext(current.getnext())
                temp.setprev(current)
                current.getnext().setprev(temp)
                current.setnext(temp)

    def pop(self,pos = -1):
        # pos是一个可缺省变量，缺省值为-1
        pos = pos % self.size()
        # 如果列表只有一个元素，那么只能删除并返回这一个元素
        if self.size() == 1:
            a = self.head.getdata()
            self.head = None
            self.tail = None
            return a
        # 否则
        else:
            # 将current指向待删除的节点
            current = self.head
            for i in range(0,pos):
                current = current.getnext()
            # 如果待删除的是第一个元素
            if current == self.head :
                next = current.getnext()
                next.setprev(None)
                self.head = next
                return current.getdata()
            # 如果待删除的是最后一个元素
            elif current == self.tail:
                prev = current.getprev()
                prev.setnext(None)
                self.tail = prev
                return current.getdata()
            # 如果待删除的是中间的元素
            else:
                prev = current.getprev()
                next = current.getnext()
                prev.setnext(next)
                next.setprev(prev)
                return current.getdata()

    def __str__(self):
        # 创建一个普通列表用来储存无序表中的元素
        mylist = []
        # 用current历遍无序表，将无序表的所有元素读入mylist
        current = self.head
        while current != None:
            mylist.append(current.getdata())
            current = current.getnext()
        # 返回mylist的字符串形式
        return(str(mylist))

    def __getitem__(self, item):

        # 判断传入的参数是否是切片格式
        # 如果是单个正整数，那么返回对应位置的列表的值
        if isinstance(item,int):
            item = item % self.size()
            current = self.head
            for i in range(0,item):
                current = current.getnext()
            return current.getdata()
        # 如果是切片形式
        elif isinstance(item,slice):
            # 创建一个新无序表储存切片中的元素
            slicelist = UnorderedList()
            # 读入切片的三个参数start，stop和step
            start = item.start % self.size()
            stop = item.stop % self.size()
            if item.step == None:
                step = 1
            else:
                step = item.step % self.size()
            # 用current历遍所有节点，如果s（current的位置）在切片请求中（range（start，stop，step）），
            # 那么将此时的current的值存入slicelist
            current = self.head
            for s in range(0,self.size()):
                if s in range(start,stop,step):
                    slicelist.append(current.getdata())
                current = current.getnext()
            return slicelist

    def remove(self,item):
        # 将current指向第一个值为item的节点
        current = self.head
        while current.getdata() != item:
            current = current.getnext()
        # 如果待删除的是第一个元素
        if current == self.head:
            next = current.getnext()
            next.setprev(None)
            self.head = next
            return current.getdata()
        # 如果待删除的是最后一个元素
        elif current == self.tail:
            prev = current.getprev()
            prev.setnext(None)
            self.tail = prev
            return current.getdata()
        # 如果待删除的是中间的元素
        else:
            prev = current.getprev()
            next = current.getnext()
            prev.setnext(next)
            next.setprev(prev)
            return current.getdata()

    def search(self,item):
        # 用current历遍列表，将current的值和item比较
        current = self.head
        # 用found来存结果
        found = False
        while current != None and not found:
            if current.getdata() == item:
                found = True
            else:
                current = current.getnext()

        return found

