#对于采用开放定址冲突解决技术（这里为线性探测）的散列表，在负载因子达到给定的阈值时，散列表能够实现自动增长（这里为倍增）
class HashTable:
    def __init__(self, size, limit):
        self.size = size                                       #...........散列表大小
        self.limit = limit                                     #...........散列表负载因子阈值
        self.slots = [None] * self.size                       #...........槽
        self.hold = 0                                          #...........已盛放数据的槽的数目

    def __len__(self):
        return self.hold

    def __str__(self):
        return str(self.slots)

    def Hash(self, item):                             #.........散列函数（求余）
        return item % self.size

    def reHash(self, oldhash):                        #.........再散列
        oldhash = (oldhash + 1) % self.size            #.........线性探测
        if self.slots[oldhash] != None:
            return self.reHash(oldhash)
        else:
            return oldhash

    def check(self):                                 #........判断是否达到负载因子上限；若达到，拓展散列表
        if self.hold / self.size >= self.limit:
            self.slots += [None] * self.size         #........散列表倍增
            self.size *= 2
            i = 0            #..........对之前散列表中已有数据的位置作出适当调整，目的是最大化地提高散列表查找的效率
            while i < self.size / 2:
                if self.slots[i] == None:        #.........跳过空槽
                    i += 1
                else:
                    pos = self.Hash(self.slots[i])          #......pos代表在拓展后的散列表中当前项的哈希值
                    if pos == i:          #.......pos和当前位置一致，不需要移动
                        i += 1
                    elif self.slots[pos] == None:      #.......pos对应的是空槽，直接将当前项放进去，同时将原槽制成空槽
                        self.slots[i], self.slots[pos] = self.slots[pos], self.slots[i]
                        i += 1
                    elif self.Hash(self.slots[pos]) == pos:       #.........pos对应的不是空槽，且该槽中的数据在新的散列表中不需要移动
                        pos = self.reHash(pos)                     #.........再散列
                        self.slots[i], self.slots[pos] = self.slots[pos], self.slots[i]
                        i += 1
                    else:      #..........pos槽中的数据也需要移动，则将pos槽中的数据取出，放入当前数据，然后处理取出的数据（i不变）
                        self.slots[i], self.slots[pos] = self.slots[pos], self.slots[i]

    def put(self, item):         #......将数据加入到散列表中
        self.hold += 1
        self.check()              #......检查负载因子是否达到阈值
        pos = self.Hash(item)          #......通过散列（再散列）确定位置
        if self.slots[pos] != None:
            pos = self.reHash(pos)
        self.slots[pos] = item       #.......入槽

    def search(self, item):         #.........散列表查找
        pos = self.Hash(item)
        found = False
        count = 0
        while not found and count < self.size:
            count += 1
            if self.slots[pos] == item:
                found = True
            else:
                pos = (pos + 1) % self.size        #......哈希值对应的槽中未找到，则顺序遍历查找，直至找到或遍历结束
        return found
