#作业要求实现无序表的append, index, pop, insert，__str__和切片方法；代码中额外包括其它方法是为便于后续调用
class Node:                          #..............节点定义（单向）
    def __init__(self, data):
        self.data = data
        self.next = None

    def getData(self):
        return self.data

    def getNext(self):
        return self.next

    def setData(self, newdata):
        self.data = newdata

    def setNext(self, newnext):
        self.next = newnext


class UnorderedList:                #.............定义无序表
    def __init__(self):
        self.head = None

    def isEmpty(self):
        return self.head == None

    def size(self):
        count = 0
        current = self.head
        while current != None:
            count += 1
            current = current.getNext()
        return count

    def index(self, item):                    #............index方法，前提须保证表中含有item
        count = 0             #...................记录位置
        stop = False
        current = self.head
        while not stop:
            if current.getData() == item:         #.............找到item，停止遍历
                stop = True
            else:
                count += 1
                current = current.getNext()
        return count                    #..................返回item在表中的位置

    def search(self, item):
        found = False
        current = self.head
        while current != None and not found:
            if current.getData() == item:
                found = True
            else:
                current = current.getNext()
        return found

    def add(self, item):
        temp = Node(item)
        temp.setNext(self.head)
        self.head = temp

    def append(self, item):        #...............append方法
        temp = Node(item)           #...............创建新节点
        if self.isEmpty():             #..................若链表为空，单独处理(初始化head)
            self.head = temp
        else:
            current = self.head
            while current.getNext() != None:        #................链表不为空，遍历至末尾
                current = current.getNext()
            current.setNext(temp)                #...............在末尾添加item

    def insert(self, pos, item):           #.....insert方法；前提需保证表中有足够多的项使item能够插入到pos位置中
        count = 0                 #..................记录位置
        previous = None                   #............previous用于记录当前节点的前一个节点，便于链表维护
        current = self.head
        while count < pos:                     #..............定位至pos
            previous = current
            current = current.getNext()
            count += 1
        temp = Node(item)             #..........创建新节点
        temp.setNext(current)                #...............初始化节点的next
        if previous == None:           #............对应pos为0的情况，单独处理head
            self.head = temp
        else:                              #........pos不为0，将新节点与previous连接起来，维护链表
            previous.setNext(temp)

    def remove(self, item):
        previous = None
        current = self.head
        while current.getData() != item:
                previous = current
                current = current.getNext()
        if previous == None:
            self.head = current.getNext()
        else:
            previous.setNext(current.getNext())

    def pop(self, pos=-1):         #.......pop()/pop(pos)方法，定义缺省值pos=-1，代表pop()的特殊情况
        if pos == -1:                       #.........调用时没有传参，将pos修正至最后一位
            pos = self.size() - 1
        count = 0
        previous = None              #............记录当前节点的前一项，便于链表维护
        current = self.head
        while count < pos:                    #............定位至pos
            previous = current
            current = current.getNext()
            count += 1
        if previous == None:              #...............pos为0，单独处理head
            self.head = current.getNext()
        else:                 #................pos不为0，前一项的next直接置成当前项的后一项
            previous.setNext(current.getNext())
        return current.getData()         # ............返回被删除项的data

    def __str__(self):                  #............__str__方法
        lst = []                #..............用于存储链表每一项的data
        current = self.head
        while current != None:                  #............遍历链表，存储data
            lst.append(current.getData())
            current = current.getNext()
        return ' '.join(list(map(str, lst)))     #...........生成字符串并返回

    def __getitem__(self, key):            #.......key为int时返回相应项的data，key为slice时行使切片功能
        if isinstance(key, int):
            count = 0
            current = self.head
            while count < key:
                current = current.getNext()
                count += 1
            return current.getData()
        elif isinstance(key, slice):
            (start, stop, step) = key.indices(self.size())    #...........取出start, stop, step信息
            count = 0
            current = self.head
            while count < start:          #...........先定位到start位置
                current = current.getNext()
                count += 1
            newUOL = UnorderedList()        #.........创建新的无序表
            while count < stop:                     #......按切片要求向新表中添加项
                newUOL.append(current.getData())
                for i in range(step):                #........模拟步长
                    current = current.getNext()
                    if current == None:                   #.............最后一步越过stop，退出模拟
                        break
                count += step
            return newUOL               #...........返回切片，类型保持一致，仍为UnorderedList
