from NewNode import Node

#基本思路一致，只是在插入项的时候要考虑同时定义两个方向，删除的时候也要同时连接两个方向。
class UnorderedList:

    def __init__(self):
        self.head = None
        self.tail = None
    
    def add(self,item):
        temp = Node(item)
        #如果是空列表，则将head和tail都指向新数据项
        if self.tail == None:
            self.head = temp
            self.tail = temp
        else:
            temp.setNext(self.head)
            self.head.setPrev(temp)
            self.head = temp #更新head，确保head总是位于表头

    def append(self,item):
        temp = Node(item)
        #如果是空列表，则将head和tail都指向新数据项
        if self.tail == None:
            self.head = temp
            self.tail = temp
        else:
            self.tail.setNext(temp)
            temp.setPrev(self.tail)
            self.tail = temp #更新tail，确保tail总是位于表尾

    def size(self):
        current = self.head
        count = 0
        while current != None:
            count = count + 1
            current = current.getNext()
        return count

    def search(self,item):
        current = self.head
        found = False
        while current != None and not found:
            if current.getData() == item:
                found = True
            else:
                current = current.getNext()
        return found

    def remove(self,item):
        current = self.head
        found = False
        while not found:
            if current.getData() == item:
                found = True
            else:
                current = current.getNext()
        if self.size() == 1:
            self.head = None
            self.tail = None
        elif current.getPrev() == None:
            self.head.getNext().setPrev(self.head.getPrev())
            self.head = self.head.getNext()
        elif current.getNext() == None:
            self.tail.getPrev().setNext(self.tail.getNext())
            self.tail = self.tail.getPrev()
        else:
            current.getPrev().setNext(current.getNext())
            current.getNext().setPrev(current.getPrev())

    def index(self,item):
        current = self.head
        count = 0
        #从链表头开始遍历列表，每过一个元素count就加1
        #直到遇到的元素即为所要查找的元素,此时count的计数就是索引值，即位置
        while current.getData() != item:
            count = count + 1
            current = current.getNext()
        return count

    def pop(self,num = None):
        if num == None:
            num = self.size() - 1
        elif num < 0:
            num = num + self.size()
        else:
            num = num
            
        if num == self.size() - 1:
            popnum = self.tail.getData()
            self.tail.getPrev().setNext(self.tail.getNext())
            self.tail = self.tail.getPrev()
        elif num == 0:
            popnum = self.head.getData()
            self.head.getNext().setPrev(self.head.getPrev())
            self.head = self.head.getNext()
        else:
            current = self.head
            count = 0
            while count < num:
                count = count + 1
                current = current.getNext()
            popnum = current.getData()
            current.getNext().setPrev(current.getPrev())
            current.getPrev().setNext(current.getNext())
        return popnum

    def insert(self,num,item):
        temp = Node(item)
        if num < -self.size() or num == 0:
            self.head.setPrev(temp)
            temp.setNext(self.head)
            self.head = temp
        elif num >= self.size():
            temp.setPrev(self.tail)
            self.tail.setNext(temp)
            self.tail = temp
        else:
            num = num % self.size()
            current = self.head
            count = 0
        #一旦找到索引值所对应的位置，就应该在current和previous之间插入item
            while count < num:
                count = count + 1
                current = current.getNext()
            current.getPrev().setNext(temp)
            temp.setPrev(current.getPrev())
            current.setPrev(temp)
            temp.setNext(current)

    def __str__(self):
        current = self.head
        ret = ''
        #本函数可以实现输入为数字、输出就不带引号或输入为字符串、输出就带引号的目的
        #这样虽然输出的是字符串，但是从表面上看里面的字符串和数字是相互区别的，‘1’和1是不同的。
        while current is not None:
            #判断如果当前值是字符串，就手动加上一对引号，再用空格与其他元素分离
            if isinstance(current.getData(),str):
                ret = ret + '\'' + current.getData() + '\'' + ' '
            #否则，就只是用空格分离不同元素
            else:
                ret = ret + str(current.getData()) + ' '
            current = current.getNext()
        return '['+','.join(ret.split())+']' #最后以形似列表的样子输出
    
    __repr__ = __str__

    #本函数中，切片加入了step参数，可以实现跳跃的切片
    def __getitem__(self,n):
        #如果输入是一个数字，就是按照索引值提取数据
        #current从表头开始移动，conut计数，当count为索引值时，current就是要提取的数据
        if isinstance(n,int):
            current = self.head
            count = 0
            while count < n:
                count = count + 1
                current = current.getNext()
            return current.getData()

        #如果输入的是一个切片，就是要提取开始和结束索引值之间的数据
        #定义切片的开始索引值为i，结束索引值为j
        #current从链表头开始移动，count计数，相当于记录current的位置
        if isinstance(n,slice):
            i = n.start
            j = n.stop
            k = n.step
            
            #i如果是没有输入，就意味着从链表头开始，即i＝0
            #i如果是负数，就意味着从链表尾开始往前数，因此加上列表长度转化为正的索引值
            if i == None or i < -self.size():
                i = 0
            elif i >= self.size():
                i = self.size()
            else:
                i = i % self.size()
            
            #j如果是没有输入，就意味着从链表头开始，即i＝0
            #j如果是负数，就意味着从链表尾开始往前数，因此加上列表长度转化为正的索引值
            if j == None or j >= self.size():
                j = self.size()
            elif j < -self.size():
                j = 0
            else:
                j = j % self.size()

            #k如果是没有输入，默认缺省值是None，为避免后面余数计算出现错误，就当k为None的时候变成1处理
            if k == None:
                k = 1
            else:
                k = k
                
            if k > 0:
                current = self.head
                count = 0
                #如果count<i,那么就不在切片范围内，不输出数据，直到count计数达到i
                while count < i:
                    count = count + 1
                    current = current.getNext()
    
                #当count达到i之后，如果count<j，说明数据在切片范围内，开始输出数据，但同时注意到step
                #利用count－i将计数初始化，初始化的计数如果可以被step整除，就说明是current需要被输出的
                #否则就不输出，只扫过，得到有跳跃的切片
                ret = ''
                while count < j:
                    if (count - i) % k == 0:
                        if isinstance(current.getData(),str):
                            ret = ret + '\'' + current.getData() + '\'' + ' '
                        else:
                            ret = ret + str(current.getData()) + ' '
                        count = count + 1
                        current = current.getNext()
                    else:
                        count = count + 1
                        current = current.getNext()

            #反向切片
            else:
                current = self.tail
                count = self.size() - 1
                while count > j - 1:
                    count = count - 1
                    current = current.getPrev()
    
                ret = ''
                while count > i - 1:
                    if (count - j + 1) % k == 0:
                        if isinstance(current.getData(),str):
                            ret = ret + '\'' + current.getData() + '\'' + ' '
                        else:
                            ret = ret + str(current.getData()) + ' '
                        count = count - 1
                        current = current.getPrev()
                    else:
                        count = count - 1
                        current = current.getPrev()
            
            return '['+','.join(ret.split())+']'
