from Node import Node

class UnorderedList:

    def __init__(self):
        self.head = None
    
    def add(self,item):
        temp = Node(item)
        temp.setNext(self.head)
        self.head = temp

    def size(self):
        current = self.head
        count = 0
        while current != None:
            count = count + 1
            current = current.getNext()
        return count

    def search(self,item):
        current = self.head
        found = False
        while current != None and not found:
            if current.getData() == item:
                found = True
            else:
                current = current.getNext()

        return found

    def remove(self,item):
        current = self.head
        previous = None
        found = False
        while not found:
            if current.getData() == item:
                found = True
            else:
                previous = current
                current = current.getNext()
                
        if previous == None:
            self.head = current.getNext()
        else:
            previous.setNext(current.getNext())

    def append(self,item):
        temp = Node(item)
        current = self.head
        previous = None
        if self.head == None:
            self.add(item) #在列表为空的时候，append相当于add
        else:
            while current != None: #从链表头开始遍历到表尾
                previous = current
                current = current.getNext()
            previous.setNext(temp) #在表尾加入新数据项

    def index(self,item):
        current = self.head
        count = 0
        #从链表头开始遍历列表，每过一个元素count就加1
        #直到遇到的元素即为所要查找的元素,此时count的计数就是索引值，即位置
        while current.getData() != item:
            count = count + 1
            current = current.getNext()
        return count

    def pop(self,num = None):
        #如果不输入参数，pop()就相当于从链表尾删除元素
        #因此把num设置成最后表尾元素对应的索引值，即链表长度－1
        if num == None:
            num = self.size() - 1
        #如果num小于0，意味着要从链表尾部开始删除元素
        #通过将num加上列表长度，可以将负的num值转化为正向的从链表头开始的索引值
        if num < 0:
            num = num + self.size()
        else:
            num = num
        #设置两个值，分别是当前值和过去值，同时遍历和移动
        #一旦找到索引值所对应的位置,current就是要删掉的值
        #把previous的下一个值设置成current的下一个值，相当于删掉current
        #但同时也要返回删掉的current
        current = self.head
        previous = None
        count = 0
        if num == 0:
            popnum = self.head.getData()
            self.head = self.head.getNext()
        else:
            while count < num:
                count = count + 1
                previous = current
                current = current.getNext()
            popnum = current.getData()
            previous.setNext(current.getNext())
        return popnum
        

    def insert(self,num,item):
        #如果num超级小，相当于在表头加入；如果num超级大，相当于在表尾加入，进行一个数据的调整
        if num < -self.size():
            num = 0
        if num >= self.size():
            num = self.size()
        else:
            num = num % self.size()

        temp = Node(item)
        current = self.head
        previous = None
        count = 0
        #一旦找到索引值所对应的位置，就应该在current和previous之间插入item
        #通过将temp的下一个值设置成current，previous的下一个值设置成temp，实现插入目的
        if num == 0:
            temp.setNext(self.head)
            self.head = temp
        else:
            while count < num:
                count = count + 1
                previous = current
                current = current.getNext()
            temp.setNext(current)
            previous.setNext(temp)

    def __str__(self):
        current = self.head
        ret = ''
        #本函数可以实现输入为数字、输出就不带引号或输入为字符串、输出就带引号的目的
        #这样虽然输出的是字符串，但是从表面上看里面的字符串和数字是相互区别的，‘1’和1是不同的。
        while current is not None:
            #判断如果当前值是字符串，就手动加上一对引号，再用空格与其他元素分离
            if isinstance(current.getData(),str):
                ret = ret + '\'' + current.getData() + '\'' + ' '
            #否则，就只是用空格分离不同元素
            else:
                ret = ret + str(current.getData()) + ' '
            current = current.getNext()
        return '['+','.join(ret.split())+']' #最后以形似列表的样子输出
    
    __repr__ = __str__

    #本函数中，切片加入了step参数，可以实现跳跃的切片
    def __getitem__(self,n):
        #如果输入是一个数字，就是按照索引值提取数据
        #current从表头开始移动，conut计数，当count为索引值时，current就是要提取的数据
        if isinstance(n,int):
            if n < 0:
                n = n + self.size()
            else:
                n = n
            current = self.head
            count = 0
            while count < n:
                count = count + 1
                current = current.getNext()
            return current.getData()

        #如果输入的是一个切片，就是要提取开始和结束索引值之间的数据
        #定义切片的开始索引值为i，结束索引值为j
        #current从链表头开始移动，count计数，相当于记录current的位置
        if isinstance(n,slice):
            i = n.start
            j = n.stop
            k = n.step
            current = self.head
            count = 0
            
            #i如果是没有输入，就意味着从链表头开始，即i＝0
            #i如果是负数，就意味着从链表尾开始往前数，因此加上列表长度转化为正的索引值
            if i == None or i < -self.size():
                i = 0
            elif i >= self.size():
                i = self.size()
            else:
                i = i % self.size()
            
            #j如果是没有输入，就意味着从链表头开始，即i＝0
            #j如果是负数，就意味着从链表尾开始往前数，因此加上列表长度转化为正的索引值
            if j == None or j >= self.size():
                j = self.size()
            elif j < -self.size():
                j = 0
            else:
                j = j % self.size()

            #k如果是没有输入，默认缺省值是None，为避免后面余数计算出现错误，就当k为None的时候变成1处理
            if k == None:
                k = 1
            else:
                k = k

            if k > 0:
                #如果count<i,那么就不在切片范围内，不输出数据，直到count计数达到i
                while count < i:
                    count = count + 1
                    current = current.getNext()
    
                #当count达到i之后，如果count<j，说明数据在切片范围内，开始输出数据，但同时注意到step
                #利用count－i将计数初始化，初始化的计数如果可以被step整除，就说明是current需要被输出的
                #否则就不输出，只扫过，得到有跳跃的切片
                ret = ''
                while count < j:
                    if (count - i) % k == 0:
                        if isinstance(current.getData(),str):
                            ret = ret + '\'' + current.getData() + '\'' + ' '
                        else:
                            ret = ret + str(current.getData()) + ' '
                        count = count + 1
                        current = current.getNext()
                    else:
                        count = count + 1
                        current = current.getNext()
                           
            return '['+','.join(ret.split())+']'
