#扩展点：定义了__str__方法，使得ADT Map的输出跟字典完全一样
class HashTable:
    #初始设置成嵌套列表，大列表中有11个空列表，将容纳单个数据项的槽扩展为容纳数据项集合
    def __init__(self):
        self.size = 11
        self.slots = [[] for i in range(self.size)]
        self.data = [[] for i in range(self.size)]

    #简单求余方法实现散列函数
    def hashfunction(self,key):
        return key % self.size

    def put(self,key,data):
        hashvalue = self.hashfunction(key) #寻找该存在的位置
        if self.slots[hashvalue] == []: #该位置为空，直接append进去
            self.slots[hashvalue].append(key)
            self.data[hashvalue].append(data)
        else:
            if key in self.slots[hashvalue]: #如果key已经在该位置，对对应的value进行替换
                self.data[hashvalue][self.slots[hashvalue].index(key)] = data
            else: #该位置不为空，但是key不在小列表中，直接append
                self.slots[hashvalue].append(key)
                self.data[hashvalue].append(data)

    #寻找数据的时候，只需要在对应位置的小列表中判断是否有key，如果有，输出对应value；没有，就输出None
    def get(self,key):
        startslot = self.hashfunction(key)
        if key in self.slots[startslot]:
            data = self.data[startslot][self.slots[startslot].index(key)]
        else:
            data = None
        return data

    #删除数据的时候，只需要在对应位置的小列表中分别删掉key和value，如果不存在，就输出Not Exist
    def __delitem__(self,key):
        startslot = self.hashfunction(key)
        keylist = self.slots[startslot]
        valuelist = self.data[startslot]
        if key in keylist:
            del valuelist[keylist.index(key)]
            del keylist[keylist.index(key)]
        else:
            print ('Not Exist')

    #长度即为把所有的小列表的长度相加
    def len(self):
        length = 0
        for i in range(self.size):
            length = length + len(self.slots[i])
        return length

    #判断key在不在，只需要判断对应位置的小列表中是否有key
    def __contains__(self,key):
        i = 0
        found = False
        while i < self.size and not found:
            if key in self.slots[i]:
                found = True
            else:
                i = i + 1
        return found

    #采用跟字典完全一样的形式进行输出，即｛key：value｝
    def __str__(self):
        ret = []
        for i in range(self.size):
            for j in range(len(self.slots[i])):
                
                if isinstance(self.slots[i][j],str):
                    keyitem = '\'' + self.slots[i][j] + '\''
                else:
                    keyitem = str(self.slots[i][j])
                    
                if isinstance(self.data[i][j],str):
                    valueitem = '\'' + self.data[i][j] + '\''
                else:
                    valueitem = str(self.data[i][j])
                    
                ret.append(':'.join([keyitem,valueitem]))
        return '{'+','.join(ret)+'}'
    __repr__ = __str__

    def __getitem__(self,key):
        return self.get(key)

    def __setitem__(self,key,data):
        return self.put(key,data)

    def __len__(self):
        return self.len()
