#扩展点：定义了__str__方法，使得ADT Map的输出跟字典完全一样
class HashTable:
    #初始条件：先定义一个11-100以内素数的列表，列表长度将按照素数列表的顺序进行自动增长
    #也就是说，如果需要增长列表长度，则长度定义为下一个素数
    #一开始先按照最小的素数，即11，来生成全部是None的列表
    def __init__(self):
        global sizelist
        sizelist = [x for x in range(11,101) if not [y for y in range(2,x) if x % y == 0]]
        self.size = sizelist[0]
        self.slots = [None] * self.size
        self.data = [None] * self.size

    #简单求余方式实现散列函数
    def hashfunction(self,key):
        return key % self.size

    #如果负载因子在列表中的比例高达80%，即None的比例小于20%，则扩展列表长度至下一个素数
    #如果负载因子在列表中的比例没有达到80%，直接＋1再散列
    def rehash(self,oldhash):
        if self.slots.count(None) / len(self.slots) > 0.2:
            return (oldhash + 1) % self.size
        else:
            self.size = sizelist[sizelist.index(self.size)+1]
            return (oldhash + 1) % self.size

    #以字典形式进行输出，即｛key：value｝
    def __str__(self):
        ret = []
        for i in range(self.size):
            if isinstance(self.slots[i],str):
                keyitem = '\'' + self.slots[i] + '\''
            else:
                keyitem = str(self.slots[i])

            if isinstance(self.data[i],str):
                valueitem = '\'' + self.data[i] + '\''
            else:
                valueitem = str(self.data[i])

            ret.append(':'.join([keyitem,valueitem]))
        return '{'+','.join(ret)+'}'               

    __repr__ = __str__

    def put(self,key,data):
        #在输入数据的时候，先找到对应hashfunction的位置
        hashvalue = self.hashfunction(key)

        #如果该位置是空的，直接加入
        if self.slots[hashvalue] == None:
            self.slots[hashvalue] = key
            self.data[hashvalue] = data
        else:
            #如果该位置非空，且已存在key，则对value进行替换
            if self.slots[hashvalue] == key:
                self.data[hashvalue] = data
            else:
                #如果位置非空，但该位置上不是key，则利用再散列函数扩充长度并向后寻找空位置
                #如果负载因子没有达到阀值，self.size=oldsize,不需要扩充长度，否则就扩充长度到下一个素数
                oldsize = self.size
                nextslot = self.rehash(hashvalue)
                for i in range(self.size-oldsize):
                    self.slots.append(None)
                    self.data.append(None)
                    i = i + 1
                
                #如果下一个不是空位置也不是key，继续寻找
                while self.slots[nextslot] != None and \
                      self.slots[nextslot] != key:
                    nextslot = self.rehash(nextslot)
                #如果向后找到空位置，则直接加入
                if self.slots[nextslot] == None:
                    self.slots[nextslot] = key
                    self.data[nextslot] = data
                #如果向后找到key，则对value进行替换
                else:
                    self.data[nextslot] = data

    #在寻找key的时候，就先把目前列表长度之前所有的素数都模一次，找到对应的几个位置，先看一下key是否在这几个位置中
    #如果不在，就从key％11的位置向后寻找，直到发现key，并输出对应的data
    def get(self,key):
        data = None
        numlist = [key % sizelist[i] for i in range(sizelist.index(self.size)+1)]
        for i in numlist:
            if self.slots[i] == key:
                data = self.data[i]
            else:
                startslot = key % 11
                stop = False
                found = False
                position = startslot
                while self.slots[position] != None and \
                      not found and not stop:
                    if self.slots[position] == key:
                        found = True
                        data = self.data[position]
                    else:
                        position = self.rehash(position)
                        if position == startslot:
                            stop = True
        return data

    #删除操作跟get操作类似，只是在找到key的时候不是输出对应的data，而是把key和data的位置都变成None
    def __delitem__(self,key):
        if key in self.slots:
            numlist = [key % sizelist[i] for i in range(sizelist.index(self.size)+1)]
            for i in numlist:
                if self.slots[i] == key:
                    self.slots[i] = None
                    self.data[i] = None
                else:
                    startslot = key % 11
                    stop = False
                    found = False
                    position = startslot
                    while self.slots[position] != None and \
                          not found and not stop:
                        if self.slots[position] == key:
                            self.slots[position] = None
                            self.data[position] = None
                        else:
                            position = self.rehash(position)
                            if position == startslot:
                                stop = True
        else:
            print('Not Exist')
    
    #长度即为列表长度减掉None的个数
    def len(self):
        return self.size - self.slots.count(None)

    #判断操作也与get操作类似，只不是不是输出data，而是输出布尔值
    def __contains__(self,key):
        found = False
        numlist = [key % sizelist[i] for i in range(sizelist.index(self.size)+1)]
        for i in numlist:
            if self.slots[i] == key:
                found = True
            else:
                startslot = key % 11
                stop = False
                position = startslot
                while self.slots[position] != None and \
                      not found and not stop:
                    if self.slots[position] == key:
                        found = True
                    else:
                        position = self.rehash(position)
                        if position == startslot:
                            stop = True
        return found
    
    def __getitem__(self,key):
        return self.get(key)

    def __setitem__(self,key,data):
        return self.put(key,data)

    def __len__(self):
        return self.len()
    

    
