#扩展点：定义了__str__方法，使得ADT Map的输出跟字典完全一样
class HashTable:
    #初始条件：先定义一个11-100以内素数的列表，列表长度将按照素数列表的顺序进行自动增长
    #也就是说，如果需要增长列表长度，则长度定义为下一个素数
    #一开始先按照最小的素数，即11，来生成全部是None的列表
    def __init__(self):
        global sizelist
        sizelist = [x for x in range(11,101) if not [y for y in range(2,x) if x % y == 0]]
        self.size = sizelist[0]
        self.slots = [None] * self.size
        self.data = [None] * self.size

    #简单求余方式实现散列函数
    def hashfunction(self,key):
        return key % self.size

    #+1再散列
    def rehash(self,oldhash):
        return (oldhash + 1) % self.size

    #以字典形式进行输出，即｛key：value｝
    def __str__(self):
        ret = []
        for i in range(self.size):
            if isinstance(self.slots[i],str):
                keyitem = '\'' + self.slots[i] + '\''
            else:
                keyitem = str(self.slots[i])

            if isinstance(self.data[i],str):
                valueitem = '\'' + self.data[i] + '\''
            else:
                valueitem = str(self.data[i])

            ret.append(':'.join([keyitem,valueitem]))
        return '{'+','.join(ret)+'}'               

    __repr__ = __str__

    #在加入数据的时候，先判断负载因子是否达到阀值80%
    #如果达到阀值，就扩充列表长度，相当于新建一个长的列表
    #把原来散列中已有的数据按照新的列表长度插入新列表，去掉原来的列表
    #然后按照新的列表长度插入新的数值
    #如果没有达到阀值，直接按照规则插入列表
    def put(self,key,data):
        if self.slots.count(None) / len(self.slots) < 0.2:
            self.size = sizelist[sizelist.index(self.size)+1]
            oldslots = self.slots
            olddata = self.data
            self.slots = [None] * self.size
            self.data = [None] * self.size
            for i in range(len(oldslots)):
                if oldslots[i] != None:
                    newhashvalue = self.hashfunction(oldslots[i])
                    if self.slots[newhashvalue] == None:
                        self.slots[newhashvalue] = oldslots[i]
                        self.data[newhashvalue] = olddata[i]
                    else:
                        nextslot = self.rehash(newhashvalue)
                        while self.slots[nextslot] != None:
                            nextslot = self.rehash(nextslot)
                        self.slots[nextslot] = oldslots[i]
                        self.data[nextslot] = olddata[i]
        else:
            self.size = self.size
        
        hashvalue = self.hashfunction(key)
        if self.slots[hashvalue] == None:
            self.slots[hashvalue] = key
            self.data[hashvalue] = data
        else:
            if self.slots[hashvalue] == key:
                self.data[hashvalue] = data
            else:
                if self.slots[hashvalue] == key:
                    self.data[hashvalue] = data
                else:
                    nextslot = self.rehash(hashvalue)
                    while self.slots[nextslot] != None and \
                          self.slots[nextslot] != key:
                        nextslot = self.rehash(nextslot)

                    if self.slots[nextslot] == None:
                        self.slots[nextslot] = key
                        self.data[nextslot] = data
                    else:
                        self.data[nextslot] = data
    
    #get方法与课件完全一样，先按照hashfunction对应的位置查找
    #如果不在，就继续往后遍历，直到找到空槽或者回到起点
    def get(self,key):
        startslot = self.hashfunction(key)
        data = None
        stop = False
        found = False
        position = startslot
        while self.slots[position] != None and \
              not found and not stop:
            if self.slots[position] == key:
                found = True
                data = self.data[position]
            else:
                position = self.rehash(position)
                if position == startslot:
                    stop = True
        return data

    #删除操作跟get操作类似，只是在找到key的时候不是输出对应的data
    #而是把key和data的位置都变成None
    def __delitem__(self,key):
        if key in self.slots:
            startslot = self.hashfunction(key)
            stop = False
            found = False
            position = startslot
            while self.slots[position] != None and \
                  not found and not stop:
                if self.slots[position] == key:
                    found = True
                    self.slots[position] = None
                    self.data[position] = None
                else:
                    position = self.rehash(position)
                    if position == startslot:
                        stop = True
        else:
            print('Not Exist')

    #长度即为列表长度减掉None的个数
    def len(self):
        return self.size - self.slots.count(None)

    #判断操作也与get操作类似，只不是不是输出data，而是输出布尔值
    def __contains__(self,key):
        startslot = self.hashfunction(key)
        stop = False
        found = False
        position = startslot
        while self.slots[position] != None and \
              not found and not stop:
            if self.slots[position] == key:
                found = True
                data = self.data[position]
            else:
                position = self.rehash(position)
                if position == startslot:
                    stop = True
        return found
    
    def __getitem__(self,key):
        return self.get(key)

    def __setitem__(self,key,data):
        return self.put(key,data)

    def __len__(self):
        return self.len()
    

    
