class Node:
    def __init__(self,initdata):
        self.data = initdata
        self.next = None
    def setdata(self,newnext):
        self.data = newnext
    def getdata(self):
        return self.data
    def setnext(self,newdata):
        self.next = newdata
    def getnext(self):
        return self.next
# 以上实现Node结构的基本功能

class UnorderedList:

    def __init__(self):
        # 创建一个链表，表头初始为None
        self.head = None

    def size(self):
        # size存列表的大小，初始为零
        size = 0
        # current历遍所有的元素，初始为head
        current = self.head
        # 只要current不是None，就将current移到下一个元素，并将size加一
        while current != None:
            current = current.getnext()
            size += 1
        # 返回size的值
        return size

    def remove(self,item):
        # 将current指向第一个待删除的元素，previous指向current的前一个元素
        current = self.head
        previous = None
        # 只要current的值不是请求的item，那么将current移到下一个元素
        while current.getdata() != item:
            previous = current
            current = current.getnext()
        # 删除current指向的元素
        # 若previous为None，那么直接对head做变更即可
        if previous == None:
            self.head = None
        # 否则，对previous的next做变更即可
        else:
            previous.setnext(current.getnext())

    def append(self,item):
        # temp为待插入的节点
        temp = Node(item)
        # 先将current指向head，previous指向None
        current = self.head
        previous = None
        # 只要current不是None，将previous指向current，再将current移到下一个位置
        while current != None:
            previous = current
            current = current.getnext()
        # 如果previous为None，那么初始的列表是空的，那么直接对head做变更即可
        if previous == None:
            self.head = temp
        # 否则最后previous指向最后一个节点，只要对previous的next做变更即可
        else:
            previous.setnext(temp)

    def index(self,item):
        # current是检验的位置初始为head，index是current的位置，初始为0
        current = self.head
        index = 0
        # 只要current的值不是请求的item，就将current移到下一个位置，并将index+1
        while current.getdata != item:
            current = current.getnext()
            index += 1
        # 返回index的值
        return index

    def pop(self,pos = -1):
        # 如果列表中只有一个元素，那么直接变更head并返回该元素的值
        if self.head.getnext() == None:
            a = self.head.getdata()
            self.head = None
            return a
        # 否则
        else:
            # 将previous初始为head
            previous = self.head
            # 将previous移到要删除的位置的前一个位置
            for i in range(0,pos-1):
                previous = previous.getnext()
                # a暂存要返回的值
                a = previous.getnext().getdata()
                # 删掉previous的下一个元素
                previous.setnext(previous.getnext().getnext())
                # 返回被删除元素的值
                return a

    def insert(self,pos,newdata):
        # temp是待插入的节点
        temp = Node(newdata)
        # 若列表开始为空，那么变更head
        if self.size() == 0:
            self.head = temp
        # 否则
        else:
            pos = pos % self.size()
            # 将previous初始为head
            previous= self.head
            # 如果待插入位置是0号位，变更head
            if pos == 0:
                temp.setnext(self.head)
                self.head = temp
            # 否则
            else:
                # 将previous移到待插入位置的前一个节点
                for i in range(0,pos-1):
                    current = previous.getnext()
                # 将temp和previous的next作变更
                temp.setnext(previous.getnext())
                previous.setnext(temp)

    def __str__(self):
        # 创建空列表mylist
        mylist = []
        # 用current历遍self，将每个元素依次读入mylist
        current = self.head
        while current != None:
            mylist.append(current.getdata())
            current = current.getnext()
        # 返回mylist的字符串形式
        return(str(mylist))

    def __getitem__(self, item):
        # 创建一个新无序表储存切片中的元素
        slicelist = UnorderedList()
        # 判断是一个元素请求还是一个切片请求
        # 若是元素请求
        if isinstance(item,int):
            item = item % self.size()
            # 将current初始为head
            current = self.head
            # 将current移到请求的元素位置
            for i in range(0,item):
                current = current.getnext()
            # 返回该元素的值
            return current.getdata()
        # 若是切片请求
        elif isinstance(item,slice):
            start = item.start % self.size()
            stop = item.stop % self.size()
            # 如果步长参数缺省，那么缺省为1
            if item.step == None:
                step = 1
            # 否则以输入为准
            else:
                step = item.step % self.size()
            # 将current初始为head，并历遍self
            current = self.head
            # s是current的位置
            for s in range(0,self.size()):
                # 如果s在切片请求中
                if s in range(start,stop,step):
                    # 那么将current的值插入切片无序表中
                    slicelist.append(current.getdata())
                current = current.getnext()
            # 返回切片无序表
            return str(slicelist)
# 有序表类（从无序表继承）
class OrderedList(UnorderedList):
    # 禁用append和insert方法
    append = None
    insert = None
    # 更新add方法
    def add(self,item):
        # 如果开始为空表，只需修改head
        if self.head == None:
            self.head = Node(item)
        # 否则
        else:
            # 将current初始为head，previous初始为None
            current = self.head
            previous = None
            # temp为待加入的节点
            temp = Node(item)
            # 将current指向第一个比插入值大的元素，如果所有元素都比插入值小，则current指向最后的None
            while current != None and current.getdata() < item:
                previous = current
                current = current.getnext()
            # 如果current最后指向None，将temp插到最后的位置（previous的后面）
            if current == None:
                previous.setnext(temp)
            # 否则
            else:
                # 如果表中只有一个比插入值大的元素，那么将temp插到head的位置
                if previous == None:
                    temp.setnext(self.head)
                    self.head = temp
                # 否则，将temp插到previous的后面
                else:
                    temp.setnext(current)
                    previous.setnext(temp)
    # 改进search方法
    def search(self,item):
        # current为检验的对象，初始为head
        current = self.head
        # found存查找结果，初始为False
        found = False
        # 只要current不是None而且还没有找到，将current移到下一个位置
        while current != None and not found:
            # 若出现请求的值，将found改为True
            if current.getdata() == item:
                found = True
            # 只要出现比请求值大的元素，返回查找结果
            elif current.getdata() > item:
                return found
            # 否则，将current移到下一个位置
            else:
                current = current.getnext()
        # 最后仍要返回found
        return found

