#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import timeit
import random
import matplotlib
import matplotlib.pyplot as plt
import pandas as pd

matplotlib.style.use('ggplot')

with open('list_index_time.txt','w+') as data1:
    data1.write("length\ttime\n")
    for i in range(100000,10000001,100000):
        t = timeit.Timer("mylist[random.randrange(%d)]"%i,"from __main__ import random,mylist")
        mylist = list(range(i))
        listtime = t.timeit(number=1000)
        data1.write("%d\t%10.3f\n" % (i,listtime))
list_time = pd.read_table("list_index_time.txt",index_col='length')
list_time.plot(marker='.',linestyle='')
plt.savefig('list_index_time.png',dpi=300)
 
with open('dict_index_time.txt','w+') as data2:
    data2.write("length\tget_time\tset_time\n")
    for i in range(100000,10000001,100000):
        mydict = {j:None for j in range(i)}
        getT = timeit.Timer("mydict[random.randrange(%d)]"%i,"from __main__ import random,mydict")
        setT = timeit.Timer("mydict[random.randrange(%d)]=1"%i,"from __main__ import random,mydict")
        get_time = getT.timeit(number=1000)
        set_time = setT.timeit(number=1000)
        data2.write("%d\t%10.3f\t%10.3f\n" % (i,get_time,set_time))
dict_time = pd.read_table("dict_index_time.txt",index_col='length')
dict_time.plot(marker='.',linestyle='')
plt.savefig('dict_index_time.png',dpi=300)
 
with open('del_time.txt','w+') as data3:
    data3.write("length\tlist_time\tdict_time\n")    
    for i in range(100000,10000001,100000):
        t = timeit.Timer("for j in range(0,%d,%d):del x[j]"%(i-1000,i//1000),"from __main__ import x")
        x = list(range(i))
        list_time = t.timeit(number=1)
        x = {j:None for j in range(i)}
        dict_time = t.timeit(number=1)
        data3.write("%d\t%10.4f\t%10.4f\n" % (i,list_time,dict_time))
del_time = pd.read_table("del_time.txt",index_col='length')
del_time.plot(marker='.',linestyle='')
plt.savefig('del_time.png',dpi=300)
 