#海龟制图模拟汉诺塔问题解决步骤（限定n <= 7)
from pythonds.basic.stack import Stack
import turtle
import time


class Pillar:           #.......定义“柱”为一个类
    def __init__(self, posIndex):       #.....posIndex为"LEFT"，"RIGHT"或"MIDDLE"
        self.plates = Stack()         #.........盘子套在柱子上用栈的结构来模拟
        if posIndex == "LEFT":            #...........self.pos代表柱子的横坐标，根据posIndex确定
            self.pos = -300
        elif posIndex == "RIGHT":
            self.pos = 300
        else:
            self.pos = 0
        self.currentHeight = -150      #........self.currentHeight表示柱子上最高的盘子的纵坐标

    def initialize(self):                    #..........“柱”的初始化（绘制）
        tpillar = turtle.Turtle()
        tpillar.hideturtle()
        tpillar.up()
        tpillar.pensize(20)
        tpillar.speed(3)
        tpillar.setheading(-90)
        tpillar.shape("circle")
        tpillar.color("silver")
        tpillar.setposition(self.pos, 150)
        tpillar.down()
        tpillar.forward(300)

    def peek(self):            #...........调取最上面的盘子
        return self.plates.peek()

    def push(self, plate):          #......套上一个盘子，高度增加20（定为20是因为绘图效果较好，盘子厚度适中且彼此紧贴）
        self.plates.push(plate)
        self.currentHeight += 20

    def pop(self):       #............取出最上面的盘子
        self.currentHeight -= 20
        return self.plates.pop()


class Plate:                #..........定义“盘子”为一个类
    def __init__(self, index):      #.......index为盘子的序号，用于确定其大小、颜色、初始位置
        self.index = index
        self.t = turtle.Turtle()

    def initialize(self, start):      #..........“盘子”的初始化（绘制）
        self.t.hideturtle()
        self.t.up()
        self.t.speed(3)
        self.t.setheading(90)
        self.t.setposition(start.pos, 20 * self.index - 150)     #........序号小的在下面
        self.t.shape("circle")
        self.t.shapesize(15 - self.index * 2, 1)      #........序号小的（靠下的）盘子越大
        colormap = ["purple", "indigo", "blue", "green", "yellow", "orange", "red"]
        self.t.color(colormap[self.index])
        self.t.showturtle()

    def move(self, des):          #........移动盘子至des柱
        self.t.goto(self.t.position()[0], 150)       #.....模拟折线路线
        self.t.goto(des.pos, 150)
        self.t.goto(des.pos, des.currentHeight)
        des.push(self)      #.......盘子入柱


def remove(num, pfr, pto, pby):      #.....递归函数
    if num == 1:           #..........递归结束条件
        pfr.pop().move(pto)     #num == 1,从pfr柱上取出顶端的盘子，移动至pto柱
    else:                                  #........递归方程
        remove(num - 1, pfr, pby, pto)
        remove(1, pfr, pto, pby)
        remove(num - 1, pby, pto, pfr)


def Hanoi(num, fr, to, by):  #主控函数，连接外部输入、预处理和内部实现；fr, to, by为"LEFT" "RIGHT" "MIDDLE"的排列
    myWin = turtle.Screen()
    myPillars = [Pillar(fr), Pillar(to), Pillar(by)]   #生成三个柱
    for i in range(3):
        myPillars[i].initialize()           #绘制柱子
    for i in range(num):
        myPillars[0].push(Plate(i))      #生成盘子并套在起始柱子上
        myPillars[0].peek().initialize(myPillars[0])       #绘制盘子
    time.sleep(1)     #移动开始前间隔一秒
    remove(num, myPillars[0], myPillars[1], myPillars[2])    #开始递归
    t0 = turtle.Turtle()             #后续处理（作者署名）
    t0.hideturtle()
    t0.up()
    t0.setposition(-50, -250)
    t0.pensize(100)
    t0.color("forest green")
    t0.down()
    t0.write("BY JEFF WANG")
    myWin.exitonclick()

#读入盘子个数num，起始、目标、借用柱子的相对位置（"LEFT""RIGHT""MIDDLE"的排列）
num = int(input("Number of plates wished to be moved:"))
fr, to, by = input("From:"), input("To:"), input("By:")
Hanoi(num, fr, to, by)
