#设定在500*500的平面内，左下角进、右下角出（最外层为顺时针），生成阶数为degree的希尔伯特曲线
import turtle
import math

#绘制曲线函数；size为曲线所处正方形单元的边长；clockwise表示是否按顺时针方向生成，为布尔变量；degree为阶数（递归层数）
#每阶图案都由其4个低阶图案按照由阶数奇偶性和时针顺序决定的规律生成
def draw_curve(size, clockwise, degree, t):
    if degree >= 0:                  #.......degree < 0时终止递归；degree等于0时表示最原始的倒“U”形曲线
        link = size * math.pow(0.5, degree + 1)   #按照观察到的规律确定degree阶图形中连接4个degree - 1阶图形的线段长
        if clockwise:         #........将逆、顺时针与±1对应，进而与之后绘制方向对称相反的角度变化相对应，简化代码
            k = -1
        else:
            k = 1
        #图形按照degree的奇偶可分为两种生成流程；两种流程的差异除了源于奇偶性对连接方法的影响外，还源于顺逆时针的影响
        #对于时钟顺序，基于degree阶图案的顺序，逐次绘出的degree - 1阶图案的顺序表现出“相反-相同-相同-相反”的演变规律
        #degree的奇偶不同时，均涉及到3次对低阶图案的连接；但连接前后是否变向、朝哪边变向的情况存在差异（后者与degree阶图案生成的时针顺序有关）
        #更具体的规律请参考代码和图例
        if degree % 2 == 0:
            draw_curve(size * 0.5, not clockwise, degree - 1, t)
            t.forward(link)
            t.setheading(t.heading() + k * 90)
            draw_curve(size * 0.5, clockwise, degree - 1, t)
            t.forward(link)
            draw_curve(size * 0.5, clockwise, degree - 1, t)
            t.setheading(t.heading() + k * 90)
            t.forward(link)
            draw_curve(size * 0.5, not clockwise, degree - 1, t)
        else:
            draw_curve(size * 0.5, not clockwise, degree - 1, t)
            t.setheading(t.heading() + k * 90)
            t.forward(link)
            draw_curve(size * 0.5, clockwise, degree - 1, t)
            t.setheading(t.heading() - k * 90)
            t.forward(link)
            t.setheading(t.heading() - k * 90)
            draw_curve(size * 0.5, clockwise, degree - 1, t)
            t.forward(link)
            t.setheading(t.heading() + k * 90)
            draw_curve(size * 0.5, not clockwise, degree - 1, t)


def Hilbert_Curve(degree, size=500):        #..........预处理与绘后处理
    myWin = turtle.Screen()
    myTurtle = turtle.Turtle()
    myTurtle.speed(0)          #...........海龟参数初始化
    myTurtle.pensize(5)
    myTurtle.color("sky blue")
    edge = size * math.pow(0.5, degree + 2)  #依据阶数确定海龟起点（左下角）的位置坐标，并移动海龟至该位置，等待绘图
    myTurtle.up()
    myTurtle.setposition(edge - size * 0.5, edge - size * 0.5)
    myTurtle.down()
    if degree % 2 == 0:           #............依据阶数奇偶性确定海龟初始朝向（始终保持与曲线行进方向一致）
        myTurtle.setheading(90)
    else:
        myTurtle.setheading(0)
    clockwise = True                 #..........默认从左下角开始，因此最外层为顺时针
    draw_curve(size, clockwise, degree, myTurtle)          #.........调用绘图函数，开始绘制曲线
    myTurtle.up()           #绘后处理（作者署名）
    myTurtle.goto(300, -300)
    myTurtle.pensize(7)
    myTurtle.color("forest green")
    myTurtle.down()
    myTurtle.write("BY JEFF WANG")
    myWin.exitonclick()

Hilbert_Curve(int(input()))   #........读入待生成Hilbert Curve的阶数；生成曲线
