# -*- coding:utf-8 -*-
# 调用之前写的测试时间的装饰器
import time
import random
from binary_search_test import time_wrapper
import sys

sys.setrecursionlimit(1000000)

# 这里不能在def中直接使用装饰器，否则进行一次递归都会调用一次装饰器，时间偏长
# @time_wrapper(1000)
def quick_sort(alist, first = 0, last = None):
    '''
    标准快速排序
    :param alist: 待排序列表
    :param first: 本次排序的起始
    :param last: 本次排序的末尾
    :return: None
    '''
    # 初始化参数
    if last is None:
        last = len(alist) - 1
    # 基本结束条件
    if first >= last:
        return alist
    # 确立中值，进行左右排序
    # 标准快排使用第一位作为中值
    split_point = alist[first]
    # 序数指针
    head = first + 1
    tail = last
    while True:
        # 对head、tail进行移动
        while head <= tail and alist[head] <= split_point:
            head += 1
        while tail >= head and alist[tail] >= split_point:
            tail -= 1
        # 如果满足调换条件，那么调换
        if head < tail:
            alist[head], alist[tail] = alist[tail], alist[head]
        else:
            # 退出循环，说明本轮排序结束
            break
    # 对分隔点（相遇点，即tail）进行替换
    if alist[first] > alist[tail]:
        alist[first], alist[tail] = alist[tail], alist[first]
    
    # 递归调用
    quick_sort(alist, first, tail - 1)
    quick_sort(alist, tail + 1, last)
    return alist
    
def r_quick_sort(alist, first = 0, last = None):
    '''
    取首尾平均数作为基准进行快速排序
    :param alist: 待排序列表
    :param first: 本次排序的起始
    :param last: 本次排序的末尾
    :return: None
    '''
    # 初始化参数
    
    if last is None:
        last = len(alist) - 1
    # 基本结束条件
    if first >= last:
        return alist
    # 计算首尾平均数
    mid = (alist[first] + alist[last]) / 2
    # 序数指针（第一个数也参与排序）
    head = first
    tail = last
    while True:
        # 对head、tail进行移动
        while head < tail and alist[head] <= mid:
            head += 1
        # 保证若发生错位，则只错一位
        while tail >= head and alist[tail] >= mid:
            tail -= 1
        # 如果满足调换条件，那么调换
        if head <= tail:
            alist[head], alist[tail] = alist[tail], alist[head]
        else:
            # 退出循环，说明本轮排序结束
            break
    
    # 结束后，head处于比mid大的值中最小的位置，tail处于比mid小的值中最大的位置
    # 递归
    r_quick_sort(alist, first, tail)
    r_quick_sort(alist, head, last)
    return alist
    
def is_sort(alist):
    '''
    判断是否分类完成的辅助函数
    '''
    return alist == list(sorted(alist))
    
if __name__ == '__main__':
    # 分别为无序和有序两种情形
    alist = [random.random() for i in range(500)]
    # alist = [i + random.random() for i in range(500)]
    # 首先验证排序有效性
    print(is_sort(alist[:]), is_sort(quick_sort(alist[:])), is_sort(r_quick_sort(alist[:])))
    # 然后开始计时
    print(time_wrapper(quick_sort, 100)(alist[:]))
    print(time_wrapper(r_quick_sort, 100)(alist[:]))
    # 除去切片操作
    print(time_wrapper(lambda x:x, 100)(alist[:]))
    # 500个数排序结果：
    #     乱序：原方法为0.3e-1，改进为1.9e-3，空函数：0.0
    #     顺序：原：0.3e-1，改进：1.8e-3，空函数：0.0
    # 1000个数排序结果：
    #     乱序：原方法为1.2e-1，改进为3.8e-3，空函数：0.0
    #     顺序：原：1.2e-1，改进：3.5e-3，空函数：0.0
    # 2000个数排序结果：
    #     乱序：原方法为4.6e-1，改进为9.4e-3，空函数：5e-7
    #     顺序：原：4.9e-1，改进：8.1e-3，空函数：5e-7
    # 基本上差2个数量级，排序总数越多越不明显
    # 顺序情况下，改进算法效率增加，非改进算法效率降低
    # 由于我的电脑运行更大的数据（比如3000）显示“Python已停止工作”，所以不能进行进一步测试