import time
import random

def binarySearch(alist, item):
    '''
    课件中的二分查找，直接抄过来，递归形式
    '''
    if len(alist) == 0:
        return False
    else:
        midpoint = len(alist) // 2
        if alist[midpoint] == item:
            return True
        else:
            if item<alist[midpoint]:
                return binarySearch(alist[:midpoint], item)
            else:
                return binarySearch(alist[midpoint+1:], item)
                
def binarySearch_noSlice(alist, item, start = 0, end = None):
    '''
    无切片的递归，范围[start,end)
    >>> binarySearch_noSlice([i for i in range(1000)],976)
    True
    '''
    
    if (end is not None) and (end - start <= 1):
        # 结束条件，当差小于等于1时可以判断
        return alist[start] == item or alist[end] == item
    else:
        # 对None进行处理，以后的递归中不使用None
        if end == None:
            end = len(alist)-1
            
        
        midpoint = (start + end) // 2
        if alist[midpoint] == item:
            return True
        else:
            if item<alist[midpoint]:
                return binarySearch_noSlice(alist, item, start, midpoint)
            else:
                return binarySearch_noSlice(alist, item, midpoint+1, end)        

                
def time_wrapper(func, num = 10000):
    '''
    老实说我从来没用过timeit，正因为我反感timeit。 为什么我反感，因为我完全有开发timeit的能力，timeit的底层我都了如指掌。
    虽说我反感timeit，但是timeit却占有大量的用户份额，之后我有个想法，不如重新开发一个属于自己思想，自己架构的timeit。
    我给了他一个霸气的名字：MingGetimeit!
    它的名字叫MingGetimeit，MingGe就是我的大名， 一看到timeit名字，就知道作者是我，知道它是国产的，让别人知道国产timeit一样做得很出色，出众！
    我是mingge 请支持国产minggetimeit，因为我们都是中国人。
    （以上玩梗）
    这个就是不需要复杂的初始化了，方便了很多，而且语句不在字符串里方便查错。
    :params func: 要测量的函数
    :params num: 要测量的参数
    :return: 平均单次执行时间
    '''
    import time
    def wrapped(*args):
        t_initial = time.time()
        for i in range(num):
            # 开始反复调用函数了
            func(*args)
        t_end = time.time()
        return (t_end - t_initial) / num
    return wrapped
    
if __name__ == '__main__':
    
    # 重定向标准输出流，以便输出到文件中。
    # 已经做过的一些实验数据可以在binary_search_result.json中查看
    # 如果将去掉这一段，并把缩进正常化，就是正常的代码
    
    # 打开一个JSON文件，记录数据
    with open('binary_search_result.json','a') as f:
        import sys
        # 备份标准输出流
        out = sys.stdout
        class out_to_file:
            def write(self, data):
                # 写入输出流
                out.write(data)
                f.write(data)
            def flush(self):
                # 刷新输出流
                # out.flush()
                pass
        sys.stdout = out_to_file()
        
        # 以下是正常的输出结果，如果去掉上面的代码，就只有控制台输出，而不添加到文件里
        print('{"test time %s": {'%(time.asctime()))
        # 随机生成一个数组，从两百万个数据中按概率取大概一半
        l = list(filter(lambda x:random.random()<0.5, (i for i in range(2000000))))
        print('"length of array":',len(l),',')
        # 分别执行10000次，看时间差异
        print('"with slice":',time_wrapper(binarySearch, 1000)(l, random.sample(range(2000000), 1)[0]),',')
        print('"no slice":',time_wrapper(binarySearch_noSlice, 1000)(l, random.sample(range(2000000), 1)[0]),',')
        # 空函数对照，减去生成随机数的时间
        print('"empty function":',time_wrapper(lambda x,y:None, 1000)(l, random.sample(range(2000000), 1)[0]))
        # 1000000数组，每次时间基本上分别是2.8e-2和2.8e-5，差1000倍。
        # 空函数为5e-7，与较快的noslice快100倍，可以忽略
        # 有时候空函数时间过小，会出现执行结果为0.0的情况
        print('}','}','\n')
    
    