#单词最小编辑距离问题
#思路：由于删除和添加的代价均为20，而复制为5，因此要尽量多地使用复制；于是问题归结于寻找源单词和目标单词的最长公共子序列（类似于导弹拦截）


#getIndex函数
#在字符串s中从位置start开始（包括start），寻找第一个等于ch的字符并返回其位置；若未找到则返回None
def getIndex(ch, start, s):
    found = False
    for i in range(start, len(s)):
        if s[i] == ch:
            found = True
            break
    if found:
        return i
    else:
        return None


#动归主体部分
#思路：
#......对源单词raw中的每一个字母，尝试寻找从首字母到当前字母的源单词片段与目标单词new的最长公共子序列（该序列须以当前字母结尾）
#......对raw中每一个字母对应建立一个列表，用于存储相应最长公共子序列中各个字母在目标单词中的位置（若没有符合要求的子序列则列表为空）
#......例如，raw = algorithm, new = alligator；则对于源单词最后一个字母m，当前最长公共子序列为'algor'，相应的列表为[0, 1, 4, 7, 8]
#状态转移关系：
#......对于源单词中第i个字母raw[i]，考察前（i-1）个字母对应的（i-1）个最长公共子序列
#......对于子序列最后一个字母，若在其在目标单词中的位置之后仍能找到raw[i]，则能够构成以raw[i]结尾的新的最长公共子序列
#......选取raw[i]对应的子序列中最长的一个并最终确定下来；于是根据前（i-1）个最优解，得到第i个最优解
#最后对与源单词中每一个字母对应的最优解进行比较，取最长，得到最终解；该解中相应的字母全部采取“复制”的方法

def DPcopyList(raw, new):
    dynamic = []                #..........dynamic为嵌套列表，用于存放与raw中每个字母相对应的共len(raw)个列表
    for i in range(len(raw)):
        newlist = []                #.........将raw[i]对应的列表newlist初始化为空
        newpos = getIndex(raw[i], 0, new)                #.........列表最后一项newpos初始化为在目标单词中raw[i]第一次出现的位置
        if newpos != None:                 #..........目标单词中存在raw[i]
            for j in range(i):                     #...........考察前（i-1）个最优解
                if dynamic[j] != []:                   #..........第j个最优解不为空时才能比较
                    index = getIndex(raw[i], dynamic[j][-1] + 1, new)
                    if index != None and len(dynamic[j]) > len(newlist):       #......raw[i]能够与第j个最优解构成新的公共子序列，且新序列长度大于当前newlist的长度
                        newlist = dynamic[j]               #......更新newlist
                        newpos = index                 #........更新newpost（raw[i]在满足构成新序列条件的前提下在目标单词中的位置）
            dynamic.append(newlist + [newpos])                #.......创建与raw[i]对应的新列表并存入dynamic
        else:               #.........raw[i]最初就不存在于目标单词中，直接添加空列表
            dynamic.append([])
    copyList = []                 #.........确立最终解，也即由raw与new的最长公共子序列中的字母在new中的位置构成的列表
    for i in range(len(dynamic)):
        if len(dynamic[i]) > len(copyList):
            copyList = dynamic[i]
    return copyList


#主控函数，输出操作流程，返回最小代价（最小距离）
def Process(raw, new):
    print("Procedures:")
    price = 0             #.......最小代价初始化
    copyList = DPcopyList(raw, new)               #.........得到new中待复制字母的位置列表
    praw = 0              #.........praw，pnew，plist分别代表raw，new，copyList中的动态位置标记；均初始化为0
    pnew = 0
    plist = 0
    while plist < len(copyList):               #........涉及复制操作的单词片段处理（复制操作中间可能还有删除、添加操作）
        if raw[praw] != new[copyList[plist]]:                #........源单词中的当前字母不是待复制列表的首项（即不复制）
            print("    Delete '%s'" % raw[praw])               #..........直接删除
            price += 20
            praw += 1
        elif raw[praw] != new[pnew]:              #........源单词中的当前字母是待复制列表的首项，但不等于目标单词中的当前字母（尚不能复制）
            print("    Insert '%s'" % new[pnew])               #.........添加目标单词中的当前字母
            price += 20
            pnew += 1
        else:               #...........满足采取复制操作的条件，进行复制
            print("    Copy '%s'" % new[copyList[plist]])
            price += 5
            praw += 1
            pnew += 1
            plist += 1
    if praw < len(raw):            #........处理上述操作后源单词末尾可能剩余的未删除的部分
        for i in range(praw, len(raw)):
            print("    Delete '%s'" % raw[i])
            price += 20
    if pnew < len(new):              #........处理上述操作后目标单词末尾可能剩余的尚未添加的部分
        for i in range(pnew, len(new)):
            print("    Insert '%s'" % new[i])
            price += 20
    return price            #.........返回最小代价（最小编辑距离）

words = input().split()
minimumPrice = Process(words[0], words[1])
print("Minimum Price: ", minimumPrice)
