#双向链表，节点中额外包含prev信息，链表包括head和tail；注释主要针对于特殊情况的处理和链表双向性的维护
class Node:
    def __init__(self, data):
        self.data = data
        self.prev = None         #.............节点的双向性
        self.next = None

    def getData(self):
        return self.data

    def getPrev(self):            #...........双向
        return self.prev

    def getNext(self):
        return self.next

    def setData(self, newdata):
        self.data = newdata

    def setPrev(self, newprev):        #............双向
        self.prev = newprev

    def setNext(self, newnext):
        self.next = newnext


class UnorderedList:
    def __init__(self):
        self.head = None
        self.tail = None            #............新增tail

    def isEmpty(self):
        return self.head == None

    def size(self):
        count = 0
        current = self.head
        while current != None:
            count += 1
            current = current.getNext()
        return count

    def index(self, item):
        count = 0
        stop = False
        current = self.head
        while not stop:
            if current.getData() == item:
                stop = True
            else:
                current = current.getNext()
                count += 1
        return count

    def search(self, item):
        found = False
        current = self.head
        while current != None and not found:
            if current.getData() == item:
                found = True
            else:
                current = current.getNext()
        return found

    def add(self, item):
        temp = Node(item)
        if self.isEmpty():             #.........单独处理向空表中添加项的情况，初始化head和tail
            self.head = temp
            self.tail = temp
        else:
            self.head.setPrev(temp)           #.............维护链表的双向性和表头
            temp.setNext(self.head)
            self.head = temp

    def append(self, item):
        temp = Node(item)
        if self.isEmpty():            #.........单独处理向空表中添加项的情况，初始化head和tail
            self.head = temp
            self.tail = temp
        else:
            self.tail.setNext(temp)           #.............维护链表的双向性和表尾
            temp.setPrev(self.tail)
            self.tail = temp

    def insert(self, pos, item):
        if pos == 0:            #.......插入至表头，交付add方法处理（包括链表最初只含有一项的特殊情况）
            self.add(item)
        else:                   #........插入位置不在表头；此时插入点的prev和next项不为None，均可以调用节点的方法
            count = 0
            current = self.head
            while count < pos:
                current = current.getNext()
                count += 1
            temp = Node(item)
            temp.setPrev(current.getPrev())        #.........插入新节点，维护链表双向性
            current.getPrev().setNext(temp)
            temp.setNext(current)
            current.setPrev(temp)

    def remove(self, item):     #.............前提须保证链表中含有item
        count = 0
        stop = False
        current = self.head
        while not stop:            #..........current定位至待删除item的位置；count记录该位置
            if current.getData() == item:
                stop = True
            else:
                current = current.getNext()
                count += 1
        if self.size() == 1:          #...........处理链表中最初只含有一项（即待删项）的特殊情况，重置head和tail
            self.head = current.getNext()
            self.tail = current.getPrev()
        elif count == 0:          #.........链表最初多于一项，且待删除项位于表头；单独重置head
            self.head = current.getNext()
            current.getNext().setPrev(current.getPrev())
        elif count == self.size() - 1:    #...........链表最初多于一项，且待删除项位于表尾；单独重置tail
            current.getPrev().setNext(current.getNext())
            self.tail = current.getPrev()
        else:  #...链表最初多于一项，且待删除项不位于表头或表尾；此时待删除项的prev和next不为None，均可以调用节点的方法
            current.getPrev().setNext(current.getNext())        #........维护链表的双向性
            current.getNext().setPrev(current.getPrev())

    def pop(self, pos=-1):    #.........pop()/pop(pos)；定义缺省值pos=-1
        current = self.head
        if self.size() == 1:     #..........链表只含一项，单独处理
            self.head = current.getNext()
            self.tail = current.getPrev()
        elif pos == 0:          #.........链表含多项，但待删除项为表头，单独处理
            self.head = current.getNext()
            current.getNext().setPrev(current.getPrev())
        elif pos == -1:         #..........链表含多项，但待删除项为表尾，单独处理
            current = self.tail
            current.getPrev().setNext(current.getNext())
            self.tail = current.getPrev()
        else:  #...链表含多项，且待删除项不位于表头或表尾；此时待删除项的prev和next不为None，均可以调用节点的方法
            count = 0
            while count < pos:               #.........定位至pos
                current = current.getNext()
                count += 1
            current.getPrev().setNext(current.getNext())    #..........维护链表的双向性
            current.getNext().setPrev(current.getPrev())
        return current.getData()          #..........返回pop值

    def __str__(self):
        lst = []
        current = self.head
        while current != None:
            lst.append(current.getData())
            current = current.getNext()
        return ' '.join(list(map(str, lst)))
